@@ -10,6 +10,8 @@ struct extract_channels_data {
 	const char* str_type;
 };
 
+void rgb_to_hsl_float_sat(float, float, float, float, float *, float *, float *);
+void hsl_to_rgb_float_sat(float, float, float, float *, float *, float *);
 void rgb_to_hsl(double, double, double, double *, double *, double *);
 void hsl_to_rgb(double, double, double, double *, double *, double *);
 void rgb_to_hsv(double, double, double, double *, double *, double *);
@@ -20,7 +22,9 @@ void LAB_to_xyz(double, double, double, double *, double *, double *);
 void xyz_to_rgb(double, double, double, double *, double *, double *);
 double BV_to_T(double BV);
 
-int equalize_cfa_fit_with_coeffs(fits *fit, double coeff1, double coeff2, int config);
+int pos_to_neg(fits *fit);
+
+int equalize_cfa_fit_with_coeffs(fits *fit, float coeff1, float coeff2, int config);
 
 gpointer extract_channels(gpointer p);
 
