@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2019 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -18,32 +18,43 @@
  * along with Siril. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <string.h>
+#include <math.h>
+
 #include "core/siril.h"
 #include "core/proto.h"
+#include "core/OS_utils.h"
 #include "algos/plateSolver.h"
 #include "algos/statistics.h"
 #include "core/undo.h"
 #include "core/processing.h"
 #include "gui/callbacks.h"
+#include "gui/image_display.h"
+#include "gui/image_interactions.h"
 #include "gui/PSF_list.h"
 #include "gui/dialogs.h"
+#include "gui/message_dialog.h"
+#include "gui/siril_preview.h"
 #include "opencv/opencv.h"
 #include "io/single_image.h"
 #include "io/sequence.h"
-#include "geometry.h"
+#include "io/image_format_fits.h"
 
-#include <string.h>
-#include <math.h>
+#include "geometry.h"
 
 /* this method rotates the image 180 degrees, useful after german mount flip.
  * fit->rx, fit->ry, fit->naxes[2] and fit->pdata[*] are required to be assigned correctly */
-static void fits_rotate_pi(fits *fit) {
-	int i, line, axis, line_size;
+static void fits_rotate_pi_ushort(fits *fit) {
+	int i, line, axis;
 	WORD *line1, *line2, *src, *dst, swap;
 
-	line_size = fit->rx * sizeof(WORD);
+	size_t line_size = fit->rx * sizeof(WORD);
 	line1 = malloc(line_size);
 	line2 = malloc(line_size);
+	if (!line1 || !line2) {
+		PRINT_ALLOC_ERR;
+		return;
+	}
 
 	for (axis = 0; axis < fit->naxes[2]; axis++) {
 		for (line = 0; line < fit->ry / 2; line++) {
@@ -79,26 +90,81 @@ static void fits_rotate_pi(fits *fit) {
 	free(line2);
 }
 
-static void mirrorx_gui(fits *fit) {
-		if (confirm_delete_wcs_keywords(fit)) {
-			set_cursor_waiting(TRUE);
-			undo_save_state(&gfit, "Processing: Mirror X");
-			mirrorx(fit, TRUE);
-			redraw(com.cvport, REMAP_ALL);
-			redraw_previews();
-			set_cursor_waiting(FALSE);
+static void fits_rotate_pi_float(fits *fit) {
+	int i, line, axis;
+	float *line1, *line2, *src, *dst, swap;
+
+	size_t line_size = fit->rx * sizeof(float);
+	line1 = malloc(line_size);
+	line2 = malloc(line_size);
+	if (!line1 || !line2) {
+		PRINT_ALLOC_ERR;
+		return;
+	}
+
+	for (axis = 0; axis < fit->naxes[2]; axis++) {
+		for (line = 0; line < fit->ry / 2; line++) {
+			src = fit->fpdata[axis] + line * fit->rx;
+			dst = fit->fpdata[axis] + (fit->ry - line - 1) * fit->rx;
+
+			memcpy(line1, src, line_size);
+			for (i = 0; i < fit->rx / 2; i++) {
+				swap = line1[i];
+				line1[i] = line1[fit->rx - i - 1];
+				line1[fit->rx - i - 1] = swap;
+			}
+			memcpy(line2, dst, line_size);
+			for (i = 0; i < fit->rx / 2; i++) {
+				swap = line2[i];
+				line2[i] = line2[fit->rx - i - 1];
+				line2[fit->rx - i - 1] = swap;
+			}
+			memcpy(src, line2, line_size);
+			memcpy(dst, line1, line_size);
 		}
+		if (fit->ry & 1) {
+			/* swap the middle line */
+			src = fit->fpdata[axis] + line * fit->rx;
+			for (i = 0; i < fit->rx / 2; i++) {
+				swap = src[i];
+				src[i] = src[fit->rx - i - 1];
+				src[fit->rx - i - 1] = swap;
+			}
+		}
+	}
+	free(line1);
+	free(line2);
+}
+
+static void fits_rotate_pi(fits *fit) {
+	if (fit->type == DATA_USHORT) {
+		fits_rotate_pi_ushort(fit);
+	} else if (fit->type == DATA_FLOAT) {
+		fits_rotate_pi_float(fit);
+	}
+	invalidate_WCS_keywords(&gfit);
+}
+
+static void mirrorx_gui(fits *fit) {
+	if (confirm_delete_wcs_keywords(fit)) {
+		set_cursor_waiting(TRUE);
+		undo_save_state(&gfit, _("Mirror X"));
+		mirrorx(fit, TRUE);
+		redraw(com.cvport, REMAP_ALL);
+		redraw_previews();
+		set_cursor_waiting(FALSE);
+	}
 }
 
 static void mirrory_gui(fits *fit) {
-		if (confirm_delete_wcs_keywords(fit)) {
-			set_cursor_waiting(TRUE);
-			undo_save_state(&gfit, "Processing: Mirror Y");
-			mirrory(fit, TRUE);
-			redraw(com.cvport, REMAP_ALL);
-			redraw_previews();
-			set_cursor_waiting(FALSE);
-		}
+	if (confirm_delete_wcs_keywords(fit)) {
+		set_cursor_waiting(TRUE);
+		undo_save_state(&gfit, _("Mirror Y"));
+		mirrory(fit, TRUE);
+		redraw(com.cvport, REMAP_ALL);
+		redraw_previews();
+		set_cursor_waiting(FALSE);
+	}
 }
 
 static void rotate_gui(fits *fit) {
@@ -117,10 +183,10 @@ static void rotate_gui(fits *fit) {
 		cropped = gtk_toggle_button_get_active(crop_rotation);
 
 		set_cursor_waiting(TRUE);
-		undo_save_state(&gfit, "Processing: Rotation (%.1lfdeg, cropped=%s)", angle,
+		undo_save_state(&gfit, _("Rotation (%.1lfdeg, cropped=%s)"), angle,
 				cropped ? "TRUE" : "FALSE");
 		verbose_rotate_image(fit, angle, interpolation, cropped);
-		update_used_memory();
+		
 		adjust_vport_size_to_image();
 		redraw(com.cvport, REMAP_ALL);
 		redraw_previews();
@@ -156,7 +222,7 @@ int verbose_resize_gaussian(fits *image, int toX, int toY, int interpolation) {
 	}
 
 	siril_log_color_message(_("Resample (%s interpolation): processing...\n"),
-			"red", str_inter);
+			"green", str_inter);
 
 	gettimeofday(&t_start, NULL);
 
@@ -197,7 +263,7 @@ int verbose_rotate_image(fits *image, double angle, int interpolation,
 	}
 
 	siril_log_color_message(
-			_("Rotation (%s interpolation, angle=%g): processing...\n"), "red",
+			_("Rotation (%s interpolation, angle=%g): processing...\n"), "green",
 			str_inter, angle);
 	gettimeofday(&t_start, NULL);
 
@@ -212,8 +278,8 @@ int verbose_rotate_image(fits *image, double angle, int interpolation,
 	return 0;
 }
 
-void mirrorx(fits *fit, gboolean verbose) {
-	int line, axis, line_size;
+static void mirrorx_ushort(fits *fit, gboolean verbose) {
+	int line, axis;
 	WORD *swapline, *src, *dst;
 	struct timeval t_start, t_end;
 
@@ -222,8 +288,12 @@ void mirrorx(fits *fit, gboolean verbose) {
 		gettimeofday(&t_start, NULL);
 	}
 
-	line_size = fit->rx * sizeof(WORD);
+	size_t line_size = fit->rx * sizeof(WORD);
 	swapline = malloc(line_size);
+	if (!swapline) {
+		PRINT_ALLOC_ERR;
+		return;
+	}
 
 	for (axis = 0; axis < fit->naxes[2]; axis++) {
 		for (line = 0; line < fit->ry / 2; line++) {
@@ -243,11 +313,54 @@ void mirrorx(fits *fit, gboolean verbose) {
 	invalidate_WCS_keywords(fit);
 }
 
+static void mirrorx_float(fits *fit, gboolean verbose) {
+	int line, axis;
+	float *swapline, *src, *dst;
+	struct timeval t_start, t_end;
+
+	if (verbose) {
+		siril_log_color_message(_("Horizontal mirror: processing...\n"), "green");
+		gettimeofday(&t_start, NULL);
+	}
+
+	size_t line_size = fit->rx * sizeof(float);
+	swapline = malloc(line_size);
+	if (!swapline) {
+		PRINT_ALLOC_ERR;
+		return;
+	}
+
+	for (axis = 0; axis < fit->naxes[2]; axis++) {
+		for (line = 0; line < fit->ry / 2; line++) {
+			src = fit->fpdata[axis] + line * fit->rx;
+			dst = fit->fpdata[axis] + (fit->ry - line - 1) * fit->rx;
+
+			memcpy(swapline, src, line_size);
+			memcpy(src, dst, line_size);
+			memcpy(dst, swapline, line_size);
+		}
+	}
+	free(swapline);
+	if (verbose) {
+		gettimeofday(&t_end, NULL);
+		show_time(t_start, t_end);
+	}
+	invalidate_WCS_keywords(fit);
+}
+
+void mirrorx(fits *fit, gboolean verbose) {
+	if (fit->type == DATA_USHORT) {
+		mirrorx_ushort(fit, verbose);
+	} else if (fit->type == DATA_FLOAT) {
+		mirrorx_float(fit, verbose);
+	}
+}
+
 void mirrory(fits *fit, gboolean verbose) {
 	struct timeval t_start, t_end;
 
 	if (verbose) {
-		siril_log_color_message(_("Vertical mirror: processing...\n"), "red");
+		siril_log_color_message(_("Vertical mirror: processing...\n"), "green");
 		gettimeofday(&t_start, NULL);
 	}
 
@@ -266,13 +379,10 @@ void mirrory(fits *fit, gboolean verbose) {
  * data is correctly written to this new area, which makes this function
  * quite dangerous to use when fit is used for something else afterwards.
  */
-int crop(fits *fit, rectangle *bounds) {
+static void crop_ushort(fits *fit, rectangle *bounds) {
 	int i, j, layer;
 	int newnbdata;
-	struct timeval t_start, t_end;
-
-	memset(&t_start, 0, sizeof(struct timeval));
-	memset(&t_end, 0, sizeof(struct timeval));
+	struct timeval t_start = { 0 }, t_end = { 0 };
 
 	if (fit == &gfit) {
 		siril_log_color_message(_("Crop: processing...\n"), "red");
@@ -304,7 +414,53 @@ int crop(fits *fit, rectangle *bounds) {
 	}
 	invalidate_stats_from_fit(fit);
 	invalidate_WCS_keywords(fit);
+}
 
+static void crop_float(fits *fit, rectangle *bounds) {
+	int i, j, layer;
+	int newnbdata;
+	struct timeval t_start = { 0 }, t_end = { 0 };
+
+	if (fit == &gfit) {
+		siril_log_color_message(_("Crop: processing...\n"), "green");
+		gettimeofday(&t_start, NULL);
+	}
+
+	newnbdata = bounds->w * bounds->h;
+	for (layer = 0; layer < fit->naxes[2]; ++layer) {
+		float *from = fit->fpdata[layer]
+				+ (fit->ry - bounds->y - bounds->h) * fit->rx + bounds->x;
+		fit->fpdata[layer] = fit->fdata + layer * newnbdata;
+		float *to = fit->fpdata[layer];
+		int stridefrom = fit->rx - bounds->w;
+
+		for (i = 0; i < bounds->h; ++i) {
+			for (j = 0; j < bounds->w; ++j) {
+				*to++ = *from++;
+			}
+			from += stridefrom;
+		}
+	}
+	fit->rx = fit->naxes[0] = bounds->w;
+	fit->ry = fit->naxes[1] = bounds->h;
+
+	if (fit == &gfit) {
+		clear_stars_list();
+		gettimeofday(&t_end, NULL);
+		show_time(t_start, t_end);
+	}
+	invalidate_stats_from_fit(fit);
+	invalidate_WCS_keywords(fit);
+}
+
+int crop(fits *fit, rectangle *bounds) {
+	if (fit->type == DATA_USHORT) {
+		crop_ushort(fit, bounds);
+	} else if (fit->type == DATA_FLOAT) {
+		crop_float(fit, bounds);
+	} else {
+		return -1;
+	}
 	return 0;
 }
 
@@ -325,7 +481,7 @@ void on_menuitem_rotation90_activate(GtkMenuItem *menuitem, gpointer user_data)
 		cropped = gtk_toggle_button_get_active(crop_rotation);
 
 		set_cursor_waiting(TRUE);
-		undo_save_state(&gfit, "Processing: Rotation (90.0deg)");
+		undo_save_state(&gfit, _("Rotation (90.0deg)"));
 		verbose_rotate_image(&gfit, 90.0, -1, cropped);	// fast rotation, no interpolation, no crop
 		adjust_vport_size_to_image();
 		redraw(com.cvport, REMAP_ALL);
@@ -346,7 +502,7 @@ void on_menuitem_rotation270_activate(GtkMenuItem *menuitem, gpointer user_data)
 		cropped = gtk_toggle_button_get_active(crop_rotation);
 
 		set_cursor_waiting(TRUE);
-		undo_save_state(&gfit, "Processing: Rotation (-90.0deg)");
+		undo_save_state(&gfit, _("Rotation (-90.0deg)"));
 		verbose_rotate_image(&gfit, 270.0, -1, cropped);// fast rotation, no interpolation, no crop
 		adjust_vport_size_to_image();
 		redraw(com.cvport, REMAP_ALL);
@@ -364,17 +520,6 @@ void on_button_rotation_close_clicked(GtkButton *button, gpointer user_data) {
 	siril_close_dialog("rotation_dialog");
 }
 
-void on_spinbutton_rotation_value_changed(GtkSpinButton *spin_button,
-		gpointer user_data) {
-	double angle = gtk_spin_button_get_value(spin_button);
-
-	angle = fmod(angle, 360);
-	while (angle < 0) {
-		angle += 360.0;
-	}
-	gtk_spin_button_set_value(spin_button, angle);
-}
-
 void on_button_rotation_ok_clicked(GtkButton *button, gpointer user_data) {
 	rotate_gui(&gfit);
 }
@@ -422,10 +567,10 @@ void on_button_resample_ok_clicked(GtkButton *button, gpointer user_data) {
 		set_cursor_waiting(TRUE);
 		int toX = round_to_int((sample[0] / 100.0) * gfit.rx);
 		int toY = round_to_int((sample[1] / 100.0) * gfit.ry);
-		undo_save_state(&gfit, "Processing: Resample (%g - %g)", sample[0] / 100.0,
+		undo_save_state(&gfit, _("Resample (%g - %g)"), sample[0] / 100.0,
 				sample[1] / 100.0);
 		verbose_resize_gaussian(&gfit, toX, toY, interpolation);
-		update_used_memory();
+		
 		adjust_vport_size_to_image();
 		redraw(com.cvport, REMAP_ALL);
 		redraw_previews();
@@ -479,15 +624,20 @@ void on_button_sample_ratio_toggled(GtkToggleButton *button, gpointer user_data)
 void on_menu_gray_crop_activate(GtkMenuItem *menuitem, gpointer user_data) {
 	// if astrometry exists
 	if (confirm_delete_wcs_keywords(&gfit)) {
-		undo_save_state(&gfit, "Processing: Crop (x=%d, y=%d, w=%d, h=%d)",
+		undo_save_state(&gfit, _("Crop (x=%d, y=%d, w=%d, h=%d)"),
 				com.selection.x, com.selection.y, com.selection.w,
 				com.selection.h);
+		if (is_preview_active()) {
+			siril_message_dialog(GTK_MESSAGE_INFO, _("A live preview session is active"),
+					_("It is impossible to crop the image when a filter with preview session is active. "
+							"Please consider to close the filter dialog first."));
+			return;
+		}
 		crop(&gfit, &com.selection);
 		delete_selected_area();
 		adjust_cutoff_from_updated_gfit();
 		redraw(com.cvport, REMAP_ALL);
 		redraw_previews();
-		update_used_memory();
 	}
 }
 
@@ -495,11 +645,58 @@ void on_menu_gray_crop_seq_activate(GtkMenuItem *menuitem, gpointer user_data) {
 	siril_open_dialog("crop_dialog");
 }
 
+int64_t crop_compute_size_hook(struct generic_seq_args *args, int nb_frames) {
+	struct crop_sequence_data *c_args = (struct crop_sequence_data*) args->user;
+	double ratio = (double)(c_args->area.h * c_args->area.w) / (double)(args->seq->rx * args->seq->ry);
+	double fullseqsize = seq_compute_size(args->seq, nb_frames, args->output_type);
+	return (int64_t)(fullseqsize * ratio);
+}
+
+int crop_image_hook(struct generic_seq_args *args, int o, int i, fits *fit,
+		rectangle *_) {
+	struct crop_sequence_data *c_args = (struct crop_sequence_data*) args->user;
+
+	return crop(fit, &(c_args->area));
+}
+
+/* TODO: should we use the partial image? */
+gpointer crop_sequence(struct crop_sequence_data *crop_sequence_data) {
+	struct generic_seq_args *args = malloc(sizeof(struct generic_seq_args));
+	args->seq = crop_sequence_data->seq;
+	args->force_float = FALSE;
+	args->partial_image = FALSE;
+	args->filtering_criterion = seq_filter_included;
+	args->nb_filtered_images = crop_sequence_data->seq->selnum;
+	args->compute_size_hook = crop_compute_size_hook;
+	args->prepare_hook = seq_prepare_hook;
+	args->finalize_hook = seq_finalize_hook;
+	args->save_hook = NULL;
+	args->image_hook = crop_image_hook;
+	args->idle_function = NULL;
+	args->stop_on_error = FALSE;
+	args->description = _("Crop Sequence");
+	args->has_output = TRUE;
+	args->output_type = get_data_type(args->seq->bitpix);
+	args->upscale_ratio = 1.0;
+	args->new_seq_prefix = crop_sequence_data->prefix;
+	args->load_new_sequence = TRUE;
+	args->force_ser_output = FALSE;
+	args->new_ser = NULL;
+	args->force_fitseq_output = FALSE;
+	args->new_fitseq = NULL;
+	args->user = crop_sequence_data;
+	args->already_in_a_thread = FALSE;
+	args->parallel = TRUE;
+
+	start_in_new_thread(generic_sequence_worker, args);
+
+	return 0;
+}
+
 /*** GUI for crop sequence */
 void on_crop_Apply_clicked(GtkButton *button, gpointer user_data) {
 	if (get_thread_run()) {
-		siril_log_message(
-				"Another task is already in progress, ignoring new request.\n");
+		PRINT_ANOTHER_THREAD_RUNNING;
 		return;
 	}
 
@@ -519,11 +716,11 @@ void on_crop_Apply_clicked(GtkButton *button, gpointer user_data) {
 	GtkEntry *cropped_entry = GTK_ENTRY(lookup_widget("cropped_entry"));
 
 	args->seq = &com.seq;
-	args->area = com.selection;
+	memcpy(&args->area, &com.selection, sizeof(rectangle));
 	args->prefix = gtk_entry_get_text(cropped_entry);
 
 	set_cursor_waiting(TRUE);
-	start_in_new_thread(crop_sequence, args);
+	crop_sequence(args);
 }
 
 void on_crop_close_clicked(GtkButton *button, gpointer user_data) {
