@@ -32,12 +32,14 @@
 #endif
 #include "core/siril.h"
 #include "core/proto.h"
-#include "gui/message_dialog.h"
+#include "core/sleef.h"
 #include "core/processing.h"
+#include "core/OS_utils.h"
 #include "gui/callbacks.h"
 #include "gui/progress_and_log.h"
 #include "gui/photometric_cc.h"
 #include "gui/dialogs.h"
+#include "gui/message_dialog.h"
 
 #ifdef HAVE_LIBCURL
 
@@ -536,7 +538,7 @@ static gchar *download_catalog(online_catalog onlineCatalog, point catalog_cente
 	char *buffer = NULL;
 	FILE *catalog = NULL;
 	FILE *fproj = NULL;
-	gchar *filename, *foutput;
+	gchar *filename, *foutput = NULL;
 
 	/* ------------------- get Vizier catalog in catalog.dat -------------------------- */
 
@@ -549,31 +551,33 @@ static gchar *download_catalog(online_catalog onlineCatalog, point catalog_cente
 		return NULL;
 	}
 	buffer = fetch_url(url);
-	fprintf(catalog, "%s", buffer);
-	g_free(url);
-	free(buffer);
-	fclose(catalog);
+	if (buffer) {
+		fprintf(catalog, "%s", buffer);
+		g_free(url);
+		free(buffer);
+		fclose(catalog);
 
-	/* -------------------------------------------------------------------------------- */
+		/* -------------------------------------------------------------------------------- */
 
-	/* --------- Project coords of Vizier catalog and save it into catalog.proj ------- */
+		/* --------- Project coords of Vizier catalog and save it into catalog.proj ------- */
 
-	foutput = g_build_filename(g_get_tmp_dir(), "catalog.proj", NULL);
-	fproj = g_fopen(foutput, "w+t");
-	if (fproj == NULL) {
-		fprintf(stderr, "plateSolver: Cannot open fproj\n");
-		g_free(foutput);
-		return NULL;
+		foutput = g_build_filename(g_get_tmp_dir(), "catalog.proj", NULL);
+		fproj = g_fopen(foutput, "w+t");
+		if (fproj == NULL) {
+			fprintf(stderr, "plateSolver: Cannot open fproj\n");
+			g_free(foutput);
+			return NULL;
+		}
+
+		convert_catalog_coords(filename, catalog_center, fproj);
+		fclose(fproj);
+
+		/* -------------------------------------------------------------------------------- */
+
+		is_result.px_cat_center = catalog_center;
+
+		g_free(filename);
 	}
-
-	convert_catalog_coords(filename, catalog_center, fproj);
-	fclose(fproj);
-
-	/* -------------------------------------------------------------------------------- */
-
-	is_result.px_cat_center = catalog_center;
-
-	g_free(filename);
 	return foutput;
 }
 
@@ -644,8 +648,8 @@ static void update_coordinates(RA ra, DEC Dec, gboolean south) {
 
 static gboolean has_any_keywords() {
 	return (gfit.focal_length > 0.0 ||
-			gfit.pixel_size_x > 0.0 ||
-			gfit.pixel_size_y > 0.0 ||
+			gfit.pixel_size_x > 0.f ||
+			gfit.pixel_size_y > 0.f ||
 			(gfit.wcs.crval1 > 0.0 && gfit.wcs.crval2 != 0.0) ||
 			(gfit.wcs.objctra[0] != '\0' && gfit.wcs.objctdec[0] != '\0'));
 }
@@ -678,12 +682,12 @@ static void update_coords() {
 static void update_pixel_size() {
 	GtkEntry *entry = GTK_ENTRY(lookup_widget("GtkEntry_IPS_pixels"));
 	gchar *cpixels;
-	double pixel;
+	float pixel;
 
 	pixel = gfit.pixel_size_x > gfit.pixel_size_y ? gfit.pixel_size_x : gfit.pixel_size_y;
 
-	if (pixel > 0.0) {
-		cpixels = g_strdup_printf("%.2lf", pixel);
+	if (pixel > 0.f) {
+		cpixels = g_strdup_printf("%.2lf", (double) pixel);
 		gtk_entry_set_text(entry, cpixels);
 		g_free(cpixels);
 	}
@@ -722,6 +726,20 @@ static void update_image_parameters_GUI() {
 	update_coords();
 }
 
+static void cd_x(wcs_info *wcs) {
+	double rot = (wcs->crota1 + wcs->crota2) / 2;
+	rot = rot * M_PI / 180.0;
+	double sinrot, cosrot;
+	double2 sc;
+	sc = xsincos(rot);
+	sinrot = sc.x;
+	cosrot = sc.y;
+	wcs->cd1_1 = wcs->cdelt1 * cosrot;
+	wcs->cd2_2 = wcs->cdelt2 * cosrot;
+	wcs->cd2_1 = wcs->cdelt1 * sinrot;
+	wcs->cd1_2 = -wcs->cdelt2 * sinrot;
+}
+
 static void update_gfit(image_solved image) {
 	gfit.focal_length = image.focal;
 	gfit.pixel_size_x = gfit.pixel_size_y = image.pixel_size;
@@ -732,8 +750,10 @@ static void update_gfit(image_solved image) {
 	gfit.wcs.equinox = 2000;
 	deg_to_HMS(image.ra, "ra", gfit.wcs.objctra);
 	deg_to_HMS(image.dec, "dec", gfit.wcs.objctdec);
-	gfit.wcs.cdelt1 = gfit.wcs.cdelt2 = image.resolution / 3600.0;
-	gfit.wcs.crota1 = gfit.wcs.crota2 = image.crota;
+	gfit.wcs.cdelt1 = image.resolution / 3600.0;
+	gfit.wcs.cdelt2 = -gfit.wcs.cdelt1;
+	gfit.wcs.crota1 = gfit.wcs.crota2 = -image.crota;
+	cd_x(&gfit.wcs);
 }
 
 static void print_platesolving_results(Homography H, image_solved image) {
@@ -745,7 +765,10 @@ static void print_platesolving_results(Homography H, image_solved image) {
 	char field_y[256] = { 0 };
 
 	/* Matching information */
-	siril_log_message(_("%d pair matches.\n"), H.pair_matched);
+	gchar *str = ngettext("%d pair match.\n", "%d pair matches.\n", H.pair_matched);
+	str = g_strdup_printf(str, H.pair_matched);
+	siril_log_message(str);
+	g_free(str);
 	inliers = 1.0 - ((((double) H.pair_matched - (double) H.Inliers)) / (double) H.pair_matched);
 	siril_log_message(_("Inliers:%*.3f\n"), 14, inliers);
 
@@ -1031,18 +1054,11 @@ static TRANS H_to_linear_TRANS(Homography H) {
 }
 
 static gboolean check_affine_TRANS_sanity(TRANS trans) {
-	gboolean ok = FALSE;
+	double var1 = fabs(trans.b) - fabs(trans.f);
+	double var2 = fabs(trans.c) - fabs(trans.e);
+	siril_debug_print("abs(b+f)=%f et abs(c+e)=%f\n", var1, var2);
 
-	double var1 = fabs(trans.b / trans.f);
-	double var2 = fabs(trans.c / trans.e);
-	siril_debug_print("abs(b/f)=%f et abs(c/e)=%f\n", var1, var2);
-
-	if (0.8 < var1 && var1 < 1.2) {
-		if (0.8 < var2 && var2 < 1.2) {
-			ok = TRUE;
-		}
-	}
-	return ok;
+	return ((fabs(var1) < 0.1) && fabs(var2) < 0.1);
 }
 
 static gboolean end_plate_solver(gpointer p) {
@@ -1080,7 +1096,7 @@ static gboolean end_plate_solver(gpointer p) {
 	g_free(args->catalogStars);
 	g_free(args->message);
 	free(args);
-	update_used_memory();
+	
 	return FALSE;
 }
 
@@ -1236,12 +1252,12 @@ static void search_object_in_catalogs(const gchar *object) {
 
 static void start_image_plate_solve() {
 	struct plate_solver_data *args = malloc(sizeof(struct plate_solver_data));
-	set_cursor_waiting(TRUE);
 
 	args->for_photometry_cc = FALSE;
-	fill_plate_solver_structure(args);
-
-	start_in_new_thread(match_catalog, args);
+	if (!fill_plate_solver_structure(args)) {
+		set_cursor_waiting(TRUE);
+		start_in_new_thread(match_catalog, args);
+	}
 }
 
 /*****
@@ -1277,7 +1293,7 @@ void on_GtkEntry_IPS_insert_text(GtkEntry *entry, const gchar *text, gint length
 	g_free(result);
 }
 
-void on_menuitem_IPS_activate(GtkMenuItem *menuitem, gpointer user_data) {
+void on_info_menu_astrometry_clicked(GtkButton *button, gpointer user_data) {
 	initialize_ips_dialog();
 	siril_open_dialog("ImagePlateSolver_Dial");
 }
@@ -1364,7 +1380,7 @@ void on_GtkCheckButton_OnlineCat_toggled(GtkToggleButton *button,
  * Public functions
  */
 
-void fill_plate_solver_structure(struct plate_solver_data *args) {
+int fill_plate_solver_structure(struct plate_solver_data *args) {
 	double fov, px_size, scale, m;
 	point catalog_center;
 
@@ -1374,13 +1390,23 @@ void fill_plate_solver_structure(struct plate_solver_data *args) {
 	m = get_mag_limit(fov);
 	catalog_center = get_center_of_catalog();
 
+	if (catalog_center.x == 0.0 && catalog_center.y == 0.0) {
+		siril_message_dialog(GTK_MESSAGE_WARNING, _("No coordinates"), _("Please enter object coordinates."));
+		return 1;
+	}
+
 	/* Filling structure */
 	args->onlineCatalog = args->for_photometry_cc ? get_photometry_catalog() : get_online_catalog(fov, m);
 	args->catalogStars = download_catalog(args->onlineCatalog, catalog_center, fov, m);
+	if (!args->catalogStars) {
+		siril_message_dialog(GTK_MESSAGE_ERROR, _("No catalog"), _("Cannot download the online star catalog."));
+		return 1;
+	}
 	args->scale = scale;
 	args->pixel_size = px_size;
 	args->manual = is_detection_manual();
 	args->fit = &gfit;
+	return 0;
 }
 
 #endif
@@ -1389,25 +1415,16 @@ gboolean confirm_delete_wcs_keywords(fits *fit) {
 	gboolean erase = TRUE;
 
 	if (fit->wcs.equinox > 0) {
-		erase = siril_confirm_dialog(_("Astrometric solution detected"), _("The astrometric solution contained in "
+		erase = siril_confirm_dialog(_("Astrometric solution detected"),
+				_("The astrometric solution contained in "
 				"the image will be erased by the geometric transformation and no undo "
-				"will be possible."), FALSE);
+				"will be possible."));
 	}
 	return erase;
 }
 
 void invalidate_WCS_keywords(fits *fit) {
 	if (fit->wcs.equinox > 0) {
-		fit->wcs.equinox = 0;
-		fit->wcs.crpix1 = 0.0;
-		fit->wcs.crpix2 = 0.0;
-		fit->wcs.crval1 = 0.0;
-		fit->wcs.crval2 = 0.0;
-		fit->wcs.cdelt1 = 0.0;
-		fit->wcs.cdelt2 = 0.0;
-		fit->wcs.crota1 = 0.0;
-		fit->wcs.crota2 = 0.0;
-		memset(fit->wcs.objctra, 0, FLEN_VALUE);
-		memset(fit->wcs.objctdec, 0, FLEN_VALUE);
+		memset(&fit->wcs, 0, sizeof(fit->wcs));
 	}
 }
