@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2019 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -40,23 +40,34 @@
 #include <gsl/gsl_statistics.h>
 #include "core/siril.h"
 #include "core/proto.h"
+#include "gui/dialogs.h"
+#include "gui/progress_and_log.h"
 #include "sorting.h"
 #include "statistics.h"
+#include "statistics_float.h"
+
+/* Activating nullcheck will treat pixels with 0 value as null and remove them
+ * from stats computation. This can be useful when a large area is black, but
+ * this shouldn't happen often. Maybe we could detect it instead of hardcoding
+ * it...
+ * Deactivating this will take less memory and make a faster statistics
+ * computation. ngoodpix will be equal to total if deactivated.
+ * Set to 0 to deactivate or 1 to activate. */
+#define ACTIVATE_NULLCHECK 0
 
 static void stats_set_default_values(imstats *stat);
 
 // copies the area of an image into the memory buffer data
-static void select_area(fits *fit, WORD *data, int layer, rectangle *bounds) {
+static void select_area_ushort(fits *fit, WORD *data, int layer, rectangle *bounds) {
 	int i, j, k = 0;
 
-	WORD *from = fit->pdata[layer] + (fit->ry - bounds->y - bounds->h) * fit->rx
-			+ bounds->x;
+	WORD *from = fit->pdata[layer] +
+		(fit->ry - bounds->y - bounds->h) * fit->rx + bounds->x;
 	int stridefrom = fit->rx - bounds->w;
 
 	for (i = 0; i < bounds->h; ++i) {
 		for (j = 0; j < bounds->w; ++j) {
-			data[k] = *from++;
-			k++;
+			data[k++] = *from++;
 		}
 		from += stridefrom;
 	}
@@ -66,36 +77,25 @@ static void select_area(fits *fit, WORD *data, int layer, rectangle *bounds) {
  * of the absolute deviations from the data's median:
  *  MAD = median (| Xi − median(X) |)
  */
-static double siril_stats_ushort_mad(WORD* data, const size_t stride,
-		const size_t n, const double m) {
+static double siril_stats_ushort_mad(WORD* data, const size_t n, const double m,
+		gboolean multithread) {
 	size_t i;
 	double mad;
 	int median = round_to_int(m);	// we use it on integer data anyway
-	WORD *tmp = calloc(n, sizeof(WORD));
-
-#pragma omp parallel for num_threads(com.max_thread) if(n > 10000) private(i) schedule(static)
-	for (i = 0; i < n; i++) {
-		int delta = data[i * stride] - median;
-		tmp[i] = (WORD)abs(delta);
+	WORD *tmp = malloc(n * sizeof(WORD));
+	if (!tmp) {
+		PRINT_ALLOC_ERR;
+		return 0.0f;
 	}
 
-	mad = histogram_median (tmp, n);
-	free(tmp);
-	return mad;
-}
-
-static double siril_stats_double_mad(const double* data, const size_t stride,
-		const size_t n, const double median) {
-	size_t i;
-	double *tmp = calloc(n, sizeof(double));
-	double mad;
-
+#ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) if(n > 10000) private(i) schedule(static)
+#endif
 	for (i = 0; i < n; i++) {
-		tmp[i] = fabs(data[i * stride] - median);
+		tmp[i] = (WORD)abs(data[i] - median);
 	}
 
-	mad = histogram_median_double (tmp, n);
+	mad = histogram_median(tmp, n, multithread);
 	free(tmp);
 	return mad;
 }
@@ -108,7 +108,9 @@ static double siril_stats_ushort_bwmv(const WORD* data, const size_t n,
 	size_t i;
 
 	if (mad > 0.0) {
+#ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) private(i) schedule(static) reduction(+:up,down)
+#endif
 		for (i = 0; i < n; i++) {
 			double yi, ai, yi2;
 
@@ -126,70 +128,8 @@ static double siril_stats_ushort_bwmv(const WORD* data, const size_t n,
 	return bwmv;
 }
 
-static double siril_stats_double_bwmv(const double* data, const size_t n,
-		const double mad, const double median) {
-
-	double bwmv = 0.0;
-	double up = 0.0, down = 0.0;
-	size_t i;
-
-	if (mad > 0.0) {
-#pragma omp parallel for num_threads(com.max_thread) private(i) schedule(static) reduction(+:up,down)
-		for (i = 0; i < n; i++) {
-			double yi, ai, yi2;
-
-			yi = (data[i] - median) / (9 * mad);
-			yi2 = yi * yi;
-			ai = (fabs(yi) < 1.0) ? 1.0 : 0.0;
-
-			up += ai * SQR(data[i] - median) * SQR(SQR (1 - yi2));
-			down += (ai * (1 - yi2) * (1 - 5 * yi2));
-		}
-		bwmv = n * (up / (down * down));
-	}
-
-	return bwmv;
-}
-
-static int IKSS(double *data, int n, double *location, double *scale) {
-	size_t i, j;
-	double mad, s, s0, m, xlow, xhigh;
-
-	quicksort_d(data, n);	// this sort is mandatory
-	i = 0;
-	j = n;
-	s0 = 1;
-	for (;;) {
-		if (j - i < 1) {
-			*location = *scale = 0;
-			break;
-		}
-		m = gsl_stats_median_from_sorted_data(data + i, 1, j - i);
-		mad = siril_stats_double_mad(data + i, 1, j - i, m);
-		s = sqrt(siril_stats_double_bwmv(data + i, j - i, mad, m));
-		if (s < 2E-23) {
-			*location = m;
-			*scale = 0;
-			break;
-		}
-		if (((s0 - s) / s) < 10E-6) {
-			*location = m;
-			*scale = 0.991 * s;
-			break;
-		}
-		s0 = s;
-		xlow = m - 4 * s;
-		xhigh = m + 4 * s;
-		while (data[i] < xlow)
-			i++;
-		while (data[j - 1] > xhigh)
-			j--;
-	}
-	return 0;
-}
-
-static WORD* reassign_to_non_null_data(WORD *data, long inputlen, long outputlen, int free_input) {
-	int i, j = 0;
+static WORD* reassign_to_non_null_data_ushort(WORD *data, size_t inputlen, size_t outputlen, int free_input) {
+	size_t i, j = 0;
 	WORD *ndata = malloc(outputlen * sizeof(WORD));
 	if (!ndata) {
 		PRINT_ALLOC_ERR;
@@ -199,8 +139,7 @@ static WORD* reassign_to_non_null_data(WORD *data, long inputlen, long outputlen
 	for (i = 0; i < inputlen; i++) {
 		if (data[i] > 0) {
 			if (j >= outputlen) {
-				//assert(0);
-				fprintf(stderr, "\n- stats MISMATCH in sizes (in: %ld, out: %ld), THIS IS A BUG: seqfile is wrong *********\n\n", inputlen, outputlen);
+				fprintf(stderr, "\n- stats MISMATCH in sizes (in: %zu, out: %zu), THIS IS A BUG: seqfile is wrong *********\n\n", inputlen, outputlen);
 				break;
 			}
 			ndata[j] = data[i];
@@ -213,7 +152,7 @@ static WORD* reassign_to_non_null_data(WORD *data, long inputlen, long outputlen
 }
 
 static void siril_stats_ushort_minmax(WORD *min_out, WORD *max_out,
-		const WORD data[], const size_t stride, const size_t n) {
+		const WORD data[], const size_t n) {
 	/* finds the smallest and largest members of a dataset */
 
 	if (n > 0 && data) {
@@ -221,13 +160,13 @@ static void siril_stats_ushort_minmax(WORD *min_out, WORD *max_out,
 		WORD max = data[0];
 		size_t i;
 
+#ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) schedule(static) if(n > 10000) reduction(max:max) reduction(min:min)
+#endif
 		for (i = 0; i < n; i++) {
-			WORD xi = data[i * stride];
-
+			WORD xi = data[i];
 			if (xi < min)
 				min = xi;
-
 			if (xi > max)
 				max = xi;
 		}
@@ -239,7 +178,8 @@ static void siril_stats_ushort_minmax(WORD *min_out, WORD *max_out,
 
 /* this function tries to get the requested stats from the passed stats,
  * computes them and stores them in it if they have not already been */
-static imstats* statistics_internal(fits *fit, int layer, rectangle *selection, int option, imstats *stats) {
+static imstats* statistics_internal_ushort(fits *fit, int layer, rectangle *selection,
+		int option, imstats *stats, gboolean multithread) {
 	int nx, ny;
 	WORD *data = NULL;
 	int stat_is_local = 0, free_data = 0;
@@ -252,14 +192,21 @@ static imstats* statistics_internal(fits *fit, int layer, rectangle *selection,
 		allocate_stats(&stat);
 		if (!stat) return NULL;
 		stat_is_local = 1;
+	} else {
+		atomic_int_incref(stat->_nb_refs);
 	}
 
 	if (fit) {
 		if (selection && selection->h > 0 && selection->w > 0) {
 			nx = selection->w;
 			ny = selection->h;
-			data = calloc(nx * ny, sizeof(WORD));
-			select_area(fit, data, layer, selection);
+			data = malloc(nx * ny * sizeof(WORD));
+			if (!data) {
+				PRINT_ALLOC_ERR;
+				if (stat_is_local) free(stat);
+				return NULL;
+			}
+			select_area_ushort(fit, data, layer, selection);
 			free_data = 1;
 		} else {
 			nx = fit->rx;
@@ -282,13 +229,13 @@ static imstats* statistics_internal(fits *fit, int layer, rectangle *selection,
 			return NULL;	// not in cache, don't compute
 		}
 		siril_debug_print("- stats %p fit %p (%d): computing minmax\n", stat, fit, layer);
-		siril_stats_ushort_minmax(&min, &max, data, 1, stat->total);
+		siril_stats_ushort_minmax(&min, &max, data, stat->total);
 		if (fit->bitpix == BYTE_IMG)
 			norm = UCHAR_MAX;
 		else norm = USHRT_MAX;
-		stat->min = (double)min;
-		stat->max = (double)max;
-		stat->normValue = (double)norm;
+		stat->min = (double) min;
+		stat->max = (double) max;
+		stat->normValue = (double) norm;
 	}
 
 	/* Calculation of ngoodpix, mean, sigma and background noise */
@@ -300,9 +247,9 @@ static imstats* statistics_internal(fits *fit, int layer, rectangle *selection,
 			return NULL;	// not in cache, don't compute
 		}
 		siril_debug_print("- stats %p fit %p (%d): computing basic\n", stat, fit, layer);
-		fits_img_stats_ushort(data, nx, ny, 1, 0, &stat->ngoodpix,
+		siril_fits_img_stats_ushort(data, nx, ny, ACTIVATE_NULLCHECK, 0, &stat->ngoodpix,
 				NULL, NULL, &stat->mean, &stat->sigma, &stat->bgnoise,
-				NULL, NULL, NULL, &status);
+				NULL, NULL, NULL, multithread, &status);
 		if (status) {
 			if (free_data) free(data);
 			if (stat_is_local) free(stat);
@@ -316,9 +263,9 @@ static imstats* statistics_internal(fits *fit, int layer, rectangle *selection,
 	}
 
 	/* we exclude 0 if some computations remain to be done or copy data if
-	 * median has to be computed */
-	if (fit && (compute_median || ((option & STATS_IKSS) && stat->total != stat->ngoodpix))) {
-		data = reassign_to_non_null_data(data, stat->total, stat->ngoodpix, free_data);
+	 * median has to be computed (this is deactivated in the ngoodpix computation) */
+	if (fit && (compute_median || (option & STATS_IKSS)) && stat->total != stat->ngoodpix) {
+		data = reassign_to_non_null_data_ushort(data, stat->total, stat->ngoodpix, free_data);
 		if (!data) {
 			if (stat_is_local) free(stat);
 			return NULL;
@@ -333,7 +280,7 @@ static imstats* statistics_internal(fits *fit, int layer, rectangle *selection,
 			return NULL;	// not in cache, don't compute
 		}
 		siril_debug_print("- stats %p fit %p (%d): computing median\n", stat, fit, layer);
-		stat->median = histogram_median(data, stat->ngoodpix);
+		stat->median = histogram_median(data, stat->ngoodpix, multithread);
 	}
 
 	/* Calculation of average absolute deviation from the median */
@@ -353,7 +300,7 @@ static imstats* statistics_internal(fits *fit, int layer, rectangle *selection,
 			return NULL;	// not in cache, don't compute
 		}
 		siril_debug_print("- stats %p fit %p (%d): computing mad\n", stat, fit, layer);
-		stat->mad = siril_stats_ushort_mad(data, 1, stat->ngoodpix, stat->median);
+		stat->mad = siril_stats_ushort_mad(data, stat->ngoodpix, stat->median, multithread);
 	}
 
 	/* Calculation of Bidweight Midvariance */
@@ -374,8 +321,8 @@ static imstats* statistics_internal(fits *fit, int layer, rectangle *selection,
 			return NULL;	// not in cache, don't compute
 		}
 		siril_debug_print("- stats %p fit %p (%d): computing ikss\n", stat, fit, layer);
-		long i;
-		double *newdata = malloc(stat->ngoodpix * sizeof(double));
+		size_t i;
+		float *newdata = malloc(stat->ngoodpix * sizeof(float));
 		if (!newdata) {
 			if (stat_is_local) free(stat);
 			if (free_data) free(data);
@@ -384,11 +331,19 @@ static imstats* statistics_internal(fits *fit, int layer, rectangle *selection,
 		}
 
 		/* we convert in the [0, 1] range */
-#pragma omp parallel for num_threads(com.max_thread) private(i) schedule(static)
+		float invertNormValue = (float)(1.0 / stat->normValue);
+#ifdef _OPENMP
+#pragma omp parallel for num_threads(com.max_thread) if (multithread) private(i) schedule(static)
+#endif
 		for (i = 0; i < stat->ngoodpix; i++) {
-			newdata[i] = (double)data[i] / stat->normValue;
+			newdata[i] = (float) data[i] * invertNormValue;
+		}
+		if (IKSS(newdata, stat->ngoodpix, &stat->location, &stat->scale, multithread)) {
+			if (stat_is_local) free(stat);
+			if (free_data) free(data);
+			free(newdata);
+			return NULL;
 		}
-		IKSS(newdata, stat->ngoodpix, &stat->location, &stat->scale);
 		/* go back to the original range */
 		stat->location *= stat->normValue;
 		stat->scale *= stat->normValue;
@@ -399,6 +354,16 @@ static imstats* statistics_internal(fits *fit, int layer, rectangle *selection,
 	return stat;
 }
 
+static imstats* statistics_internal(fits *fit, int layer, rectangle *selection, int option, imstats *stats, gboolean multithread) {
+	if (fit) {
+		if (fit->type == DATA_USHORT)
+			return statistics_internal_ushort(fit, layer, selection, option, stats, multithread);
+		if (fit->type == DATA_FLOAT)
+			return statistics_internal_float(fit, layer, selection, option, stats, multithread);
+	}
+	return statistics_internal_ushort(fit, layer, selection, option, stats, multithread);
+}
+
 /* Computes statistics on the given layer of the given opened image.
  * Mean, sigma and noise are computed with a cfitsio function rewritten here.
  * Min and max value, average deviation, MAD, Bidweight Midvariance and IKSS
@@ -413,23 +378,23 @@ static imstats* statistics_internal(fits *fit, int layer, rectangle *selection,
  * function because of the special rule of this object that has a reference
  * counter because it can be referenced in 3 different places.
  */
-imstats* statistics(sequence *seq, int image_index, fits *fit, int layer, rectangle *selection, int option) {
+imstats* statistics(sequence *seq, int image_index, fits *fit, int layer, rectangle *selection, int option, gboolean multithread) {
 	imstats *oldstat = NULL, *stat;
 	if (selection && selection->h > 0 && selection->w > 0) {
 		// we have a selection, don't store anything
-		return statistics_internal(fit, layer, selection, option, NULL);
+		return statistics_internal(fit, layer, selection, option, NULL, multithread);
 	} else if (!seq || image_index < 0) {
 		// we have a single image, store in the fits
 		if (fit->stats && fit->stats[layer]) {
 			oldstat = fit->stats[layer];
-			oldstat->_nb_refs++;
+			atomic_int_incref(oldstat->_nb_refs);
 		}
-		stat = statistics_internal(fit, layer, NULL, option, oldstat);
+		stat = statistics_internal(fit, layer, NULL, option, oldstat, multithread);
 		if (!stat) {
 			fprintf(stderr, "- stats failed for fit %p (%d)\n", fit, layer);
 			if (oldstat) {
 				stats_set_default_values(oldstat);
-				oldstat->_nb_refs--;
+				atomic_int_decref(oldstat->_nb_refs);
 			}
 			return NULL;
 		}
@@ -441,16 +406,16 @@ imstats* statistics(sequence *seq, int image_index, fits *fit, int layer, rectan
 		if (seq->stats && seq->stats[layer]) {
 			oldstat = seq->stats[layer][image_index];
 			if (oldstat)	// can be NULL here
-				oldstat->_nb_refs++;
+				atomic_int_incref(oldstat->_nb_refs);
 		}
-		stat = statistics_internal(fit, layer, NULL, option, oldstat);
+		stat = statistics_internal(fit, layer, NULL, option, oldstat, multithread);
 		if (!stat) {
 			if (fit)
 				fprintf(stderr, "- stats failed for %d in seq (%d)\n",
 						image_index, layer);
 			if (oldstat) {
 				stats_set_default_values(oldstat);
-				oldstat->_nb_refs--;
+				atomic_int_decref(oldstat->_nb_refs);
 			}
 			return NULL;
 		}
@@ -462,7 +427,7 @@ imstats* statistics(sequence *seq, int image_index, fits *fit, int layer, rectan
 	}
 }
 
-int compute_means_from_flat_cfa(fits *fit, double mean[4]) {
+int compute_means_from_flat_cfa_ushort(fits *fit, float mean[4]) {
 	int row, col, c, i = 0;
 	WORD *data;
 	unsigned int width, height;
@@ -489,26 +454,39 @@ int compute_means_from_flat_cfa(fits *fit, double mean[4]) {
 	/* Compute mean of each channel */
 	for (row = starty; row < height - 1 - starty; row += 2) {
 		for (col = startx; col < width - 1 - startx; col += 2) {
-			mean[0] += (double) data[col + row * width];
-			mean[1] += (double) data[1 + col + row * width];
-			mean[2] += (double) data[col + (1 + row) * width];
-			mean[3] += (double) data[1 + col + (1 + row) * width];
+			mean[0] += (float) data[col + row * width];
+			mean[1] += (float) data[1 + col + row * width];
+			mean[2] += (float) data[col + (1 + row) * width];
+			mean[3] += (float) data[1 + col + (1 + row) * width];
 			i++;
 		}
 	}
 
 	for (c = 0; c < 4; c++) {
-		mean[c] /= (double) i;
+		mean[c] /= (float) i;
 	}
 	return 0;
 }
 
+int compute_means_from_flat_cfa(fits *fit, float mean[4]) {
+	if (fit->type == DATA_USHORT)
+		return compute_means_from_flat_cfa_ushort(fit, mean);
+	if (fit->type == DATA_FLOAT)
+		return compute_means_from_flat_cfa_float(fit, mean);
+	return -1;
+}
+
 /****************** statistics caching and data management *****************/
 
 /* reference an imstats struct to a fits, creates the stats array if needed */
 void add_stats_to_fit(fits *fit, int layer, imstats *stat) {
-	if (!fit->stats)
+	if (!fit->stats) {
 		fit->stats = calloc(fit->naxes[2], sizeof(imstats *));
+		if (!fit->stats) {
+			PRINT_ALLOC_ERR;
+			return;
+		}
+	}
 	if (fit->stats[layer]) {
 		if (fit->stats[layer] != stat) {
 			siril_debug_print("- stats %p in fit %p (%d) is being replaced\n", fit->stats[layer], fit, layer);
@@ -516,15 +494,26 @@ void add_stats_to_fit(fits *fit, int layer, imstats *stat) {
 		} else return;
 	}
 	fit->stats[layer] = stat;
-	stat->_nb_refs++;
+	atomic_int_incref(stat->_nb_refs);
 	siril_debug_print("- stats %p saved to fit %p (%d)\n", stat, fit, layer);
 }
 
 static void add_stats_to_stats(sequence *seq, int nb_layers, imstats ****stats, int image_index, int layer, imstats *stat) {
-	if (!*stats)
+	if (!*stats) {
 		*stats = calloc(nb_layers, sizeof(imstats **));
-	if (!(*stats)[layer])
+		if (!*stats) {
+			PRINT_ALLOC_ERR;
+			return;
+		}
+	}
+	if (!(*stats)[layer]) {
 		(*stats)[layer] = calloc(seq->number, sizeof(imstats *));
+		if (!(*stats)[layer]) {
+			PRINT_ALLOC_ERR;
+			return;
+		}
+	}
+
 	if ((*stats)[layer][image_index]) {
 		if ((*stats)[layer][image_index] != stat) {
 			siril_debug_print("- stats %p, %d in seq (%d) is being replaced\n", (*stats)[layer][image_index], image_index, layer);
@@ -534,7 +523,7 @@ static void add_stats_to_stats(sequence *seq, int nb_layers, imstats ****stats,
 	siril_debug_print("- stats %p, %d in seq (%d): saving data\n", stat, image_index, layer);
 	(*stats)[layer][image_index] = stat;
 	seq->needs_saving = TRUE;
-	stat->_nb_refs++;
+	atomic_int_incref(stat->_nb_refs);
 }
 
 void add_stats_to_seq(sequence *seq, int image_index, int layer, imstats *stat) {
@@ -580,6 +569,7 @@ void invalidate_stats_from_fit(fits *fit) {
 			fit->stats[layer] = NULL;
 		}
 	}
+	fit->maxi = -1;
 }
 
 /* if image data and image structure has changed, invalidate the complete stats data structure */
@@ -607,7 +597,7 @@ void allocate_stats(imstats **stat) {
 			*stat = malloc(sizeof(imstats));
 		if (!*stat) { PRINT_ALLOC_ERR; return; } // OOM
 		stats_set_default_values(*stat);
-		(*stat)->_nb_refs = 1;
+		(*stat)->_nb_refs = atomic_int_alloc();
 		siril_debug_print("- stats %p allocated\n", *stat);
 	}
 }
@@ -615,13 +605,14 @@ void allocate_stats(imstats **stat) {
 /* frees an imstats struct if there are no more references to it.
  * returns NULL if it was freed, the argument otherwise. */
 imstats* free_stats(imstats *stat) {
-	if (stat && stat->_nb_refs-- == 1) {
+	int n;
+	if (stat && (n = atomic_int_decref(stat->_nb_refs)) == 0) {
 		siril_debug_print("- stats %p has no more refs, freed\n", stat);
 		free(stat);
 		return NULL;
 	}
 	if (stat)
-		siril_debug_print("- stats %p has refs (%d)\n", stat, stat->_nb_refs);
+		siril_debug_print("- stats %p has refs (%d)\n", stat, n);
 	return stat;
 }
 
@@ -638,3 +629,19 @@ void clear_stats(sequence *seq, int layer) {
 		}
 	}
 }
+
+/**** callbacks *****/
+
+void on_menu_gray_stat_activate(GtkMenuItem *menuitem, gpointer user_data) {
+	set_cursor_waiting(TRUE);
+	computeStat();
+	siril_open_dialog("StatWindow");
+	set_cursor_waiting(FALSE);
+}
+
+void on_info_menu_statistics_clicked(GtkButton *button, gpointer user_data) {
+	set_cursor_waiting(TRUE);
+	computeStat();
+	siril_open_dialog("StatWindow");
+	set_cursor_waiting(FALSE);
+}
