@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2019 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -36,7 +36,10 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
+#include "core/siril_app_dirs.h"
+#include "core/exif.h"
 #include "io/conversion.h"
+#include "io/ser.h"
 #include "io/sequence.h"
 #include "gui/callbacks.h"
 #include "gui/progress_and_log.h"
@@ -51,8 +54,8 @@ int round_to_int(double x) {
 	if (x <= INT_MIN + 0.5) return INT_MIN;
 	if (x >= INT_MAX - 0.5) return INT_MAX;
 	if (x >= 0.0)
-		return (int) (x + 0.5);
-	return (int) (x - 0.5);
+		return (int)(x + 0.5);
+	return (int)(x - 0.5);
 }
 
 /**
@@ -64,8 +67,8 @@ int roundf_to_int(float x) {
 	if (x <= INT_MIN + 0.5f) return INT_MIN;
 	if (x >= INT_MAX - 0.5f) return INT_MAX;
 	if (x >= 0.0f)
-		return (int) (x + 0.5f);
-	return (int) (x - 0.5f);
+		return (int)(x + 0.5f);
+	return (int)(x - 0.5f);
 }
 
 /**
@@ -75,10 +78,10 @@ int roundf_to_int(float x) {
  */
 WORD round_to_WORD(double x) {
 	if (x <= 0.0)
-		return (WORD) 0;
+		return (WORD)0;
 	if (x > USHRT_MAX_DOUBLE)
 		return USHRT_MAX;
-	return (WORD) (x + 0.5);
+	return (WORD)(x + 0.5);
 }
 
 /**
@@ -88,10 +91,32 @@ WORD round_to_WORD(double x) {
  */
 BYTE round_to_BYTE(double x) {
 	if (x <= 0.0)
-		return (BYTE) 0;
+		return (BYTE)0;
 	if (x > UCHAR_MAX_DOUBLE)
 		return UCHAR_MAX;
-	return (BYTE) (x + 0.5);
+	return (BYTE)(x + 0.5);
+}
+
+/**
+ * Round float value to a BYTE
+ * @param f value to round
+ * @return a truncated and rounded BYTE
+ */
+BYTE roundf_to_BYTE(float f) {
+	if (f <= 0.5f) return 0;
+	if (f >= UCHAR_MAX - 0.5f) return UCHAR_MAX;
+	return (BYTE)(f + 0.5f);
+}
+
+/**
+ * Round float value to a WORD
+ * @param f value to round
+ * @return a truncated and rounded WORD
+ */
+WORD roundf_to_WORD(float f) {
+	if (f <= 0.5f) return 0;
+	if (f >= USHRT_MAX - 0.5f) return USHRT_MAX;
+	return (WORD)(f + 0.5f);
 }
 
 /**
@@ -101,11 +126,11 @@ BYTE round_to_BYTE(double x) {
  */
 BYTE conv_to_BYTE(double x) {
 	if (x == 0.0)
-		return (BYTE) 0;
+		return (BYTE)0;
 	if (x == USHRT_MAX_DOUBLE)
 		return UCHAR_MAX;
 	x = ((x / USHRT_MAX_DOUBLE) * UCHAR_MAX_DOUBLE);
-	return((BYTE) (x));
+	return((BYTE)(x));
 }
 
 /**
@@ -119,6 +144,334 @@ int truncate_to_int32(uint64_t x) {
 	return (int)x;
 }
 
+WORD truncate_to_WORD(int x) {
+	if (x < 0)
+		return 0;
+	if (x > USHRT_MAX)
+		return USHRT_MAX;
+	return (WORD)x;
+}
+
+/**
+ * Set a float value in the interval given by [low, high]
+ * @param val value to be checked
+ * @param low low value of the interval
+ * @param high high value of the interval
+ * @return a new value set in the [low, high] interval
+ */
+float set_float_in_interval(float val, float low, float high) {
+	return max(low, min(val, high));
+}
+
+/**
+ * Set a double value in the interval given by [low, high]
+ * @param val value to be checked
+ * @param low low value of the interval
+ * @param high high value of the interval
+ * @return a new value set in the [low, high] interval
+ */
+double set_double_in_interval(double val, double low, double high) {
+	return max(low, min(val, high));
+}
+
+/**
+ * convert an unsigned short value to siril's representation of float values [0, 1]
+ * @param w value to convert
+ * @return the float equivalent
+ */
+float ushort_to_float_range(WORD w) {
+	return (float)w * INV_USHRT_MAX_SINGLE;
+}
+
+/**
+ * convert an unsigned char value to siril's representation of float values [0, 1]
+ * @param w value to convert
+ * @return the float equivalent
+ */
+float uchar_to_float_range(BYTE w) {
+	return (float)w * INV_UCHAR_MAX_SINGLE;
+}
+
+/**
+ * convert an double value from the unsigned short range to siril's representation
+ * of float values [0, 1]
+ * @param d value to convert
+ * @return the float equivalent
+ */
+float double_ushort_to_float_range(double d) {
+	return (float)d * INV_USHRT_MAX_SINGLE;
+}
+
+/**
+ * convert a siril float [0, 1] to an unsigned short
+ * @param f value to convert
+ * @return the unsigned short equivalent
+ */
+WORD float_to_ushort_range(float f) {
+	return roundf_to_WORD(f * USHRT_MAX_SINGLE);
+}
+
+/**
+ * convert a siril float [0, 1] to an unsigned char
+ * @param f value to convert
+ * @return the unsigned char equivalent
+ */
+BYTE float_to_uchar_range(float f) {
+	return roundf_to_BYTE(f * UCHAR_MAX_SINGLE);
+}
+
+/**
+ * convert the pixel value of an image to a float [0, 1] normalized using bitpix
+ * value depending on btpix
+ * @param fit the image the data is from
+ * @return a float [0, 1] value for the given integer value
+ */
+float ushort_to_float_bitpix(fits *fit, WORD value) {
+	float fval = (float)value;
+	return fit->orig_bitpix == BYTE_IMG ?
+		fval * INV_UCHAR_MAX_SINGLE :
+		fval * INV_USHRT_MAX_SINGLE;
+}
+
+/**
+ * convert a float type buffer into a WORD buffer
+ * @param buffer in float
+ * @param ndata
+ * @return
+ */
+WORD *float_buffer_to_ushort(float *buffer, size_t ndata) {
+	WORD *buf = malloc(ndata * sizeof(WORD));
+	if (!buf) {
+		PRINT_ALLOC_ERR;
+	} else {
+		for (size_t i = 0; i < ndata; i++) {
+			buf[i] = float_to_ushort_range(buffer[i]);
+		}
+	}
+	return buf;
+}
+
+/**
+ * convert a BYTE type buffer into a float buffer
+ * @param buffer in BYTE
+ * @param ndata
+ * @return
+ */
+float *uchar_buffer_to_float(BYTE *buffer, size_t ndata) {
+	float *buf = malloc(ndata * sizeof(float));
+	if (!buf) {
+		PRINT_ALLOC_ERR;
+	} else {
+		for (size_t i = 0; i < ndata; i++) {
+			buf[i] = uchar_to_float_range(buffer[i]);
+		}
+	}
+	return buf;
+}
+
+/**
+ * convert a WORD type buffer into a float buffer
+ * @param buffer in WORD
+ * @param ndata
+ * @return
+ */
+float *ushort_buffer_to_float(WORD *buffer, size_t ndata) {
+	float *buf = malloc(ndata * sizeof(float));
+	if (!buf) {
+		PRINT_ALLOC_ERR;
+	} else {
+		for (size_t i = 0; i < ndata; i++) {
+			buf[i] = ushort_to_float_range(buffer[i]);
+		}
+	}
+	return buf;
+}
+
+/**
+ * convert a WORD type buffer representing 8bit data into a float buffer
+ * @param buffer in WORD
+ * @param ndata
+ * @return
+ */
+float *ushort8_buffer_to_float(WORD *buffer, size_t ndata) {
+	float *buf = malloc(ndata * sizeof(float));
+	if (!buf) {
+		PRINT_ALLOC_ERR;
+	} else {
+		for (size_t i = 0; i < ndata; i++) {
+			buf[i] = uchar_to_float_range((BYTE) buffer[i]);
+		}
+	}
+	return buf;
+}
+
+/**
+ * change endianness of a 16 bit unsigned int
+ * @param x value to convert
+ * @return byte-swapped value
+ */
+uint16_t change_endianness16(uint16_t x) {
+    return (x >> 8) | (x << 8);
+}
+
+/**
+ * convert a 16 bit unsigned int in CPU byte order to little endian
+ * @param x value to convert
+ * @return little endian value
+ */
+uint16_t cpu_to_le16(uint16_t x) {
+#ifdef __BIG_ENDIAN__
+    return change_endianness16(x);
+#else
+    return x;
+#endif
+}
+
+/**
+ * convert a 16 bit unsigned int in CPU byte order to big endian
+ * @param x value to convert
+ * @return big endian value
+ */
+uint16_t cpu_to_be16(uint16_t x) {
+#ifdef __BIG_ENDIAN__
+    return x;
+#else
+    return change_endianness16(x);
+#endif
+}
+
+/**
+ * convert a 16 bit unsigned int from little endian to CPU byte order
+ * @param x little endian value to convert
+ * @return value
+ */
+uint16_t le16_to_cpu(uint16_t x) {
+    return cpu_to_le16(x);
+}
+
+/**
+ * convert a 16 bit unsigned int from big endian to CPU byte order
+ * @param x big endian value to convert
+ * @return value
+ */
+uint16_t be16_to_cpu(uint16_t x) {
+    return cpu_to_be16(x);
+}
+
+/**
+ * change endianness of a 32 bit unsigned int
+ * @param x value to convert
+ * @return byte-swapped value
+ */
+uint32_t change_endianness32(uint32_t x) {
+    return (x >> 24) | ((x & 0xFF0000) >> 8) | ((x & 0xFF00) << 8) | (x << 24);
+}
+
+/**
+ * convert a 32 bit unsigned int in CPU byte order to little endian
+ * @param x value to convert
+ * @return little endian value
+ */
+uint32_t cpu_to_le32(uint32_t x) {
+#ifdef __BIG_ENDIAN__
+    return change_endianness32(x);
+#else
+    return x;
+#endif
+}
+
+/**
+ * convert a 32 bit unsigned int in CPU byte order to big endian
+ * @param x value to convert
+ * @return big endian value
+ */
+uint32_t cpu_to_be32(uint32_t x) {
+#ifdef __BIG_ENDIAN__
+    return x;
+#else
+    return change_endianness32(x);
+#endif
+}
+
+/**
+ * convert a 32 bit unsigned int from little endian to CPU byte order
+ * @param x little endian value to convert
+ * @return value
+ */
+uint32_t le32_to_cpu(uint32_t x) {
+    return cpu_to_le32(x);
+}
+
+/**
+ * convert a 32 bit unsigned int from big endian to CPU byte order
+ * @param x big endian value to convert
+ * @return value
+ */
+uint32_t be32_to_cpu(uint32_t x) {
+    return cpu_to_be32(x);
+}
+
+/**
+ * change endianness of a 64 bit unsigned int
+ * @param x value to convert
+ * @return byte-swapped value
+ */
+uint64_t change_endianness64(uint64_t x) {
+    return
+        (x >> 56)
+        | ((x & 0xFF000000000000) >> 40)
+        | ((x & 0xFF0000000000) >> 24)
+        | ((x & 0xFF00000000) >> 8)
+        | ((x & 0xFF000000) << 8)
+        | ((x & 0xFF0000) << 24)
+        | ((x & 0xFF00) << 40)
+        | (x << 56);
+}
+
+/**
+ * convert a 64 bit unsigned int in CPU byte order to little endian
+ * @param x value to convert
+ * @return little endian value
+ */
+uint64_t cpu_to_le64(uint64_t x) {
+#ifdef __BIG_ENDIAN__
+    return change_endianness64(x);
+#else
+    return x;
+#endif
+}
+
+/**
+ * convert a 64 bit unsigned int in CPU byte order to big endian
+ * @param x value to convert
+ * @return big endian value
+ */
+uint64_t cpu_to_be64(uint64_t x) {
+#ifdef __BIG_ENDIAN__
+    return x;
+#else
+    return change_endianness64(x);
+#endif
+}
+
+/**
+ * convert a 64 bit unsigned int from little endian to CPU byte order
+ * @param x little endian value to convert
+ * @return value
+ */
+uint64_t le64_to_cpu(uint64_t x) {
+    return cpu_to_le64(x);
+}
+
+/**
+ * convert a 64 bit unsigned int from big endian to CPU byte order
+ * @param x big endian value to convert
+ * @return value
+ */
+uint64_t be64_to_cpu(uint64_t x) {
+    return cpu_to_be64(x);
+}
+
 /**
  * Test if fit has 3 channels
  * @param fit input FITS image
@@ -186,6 +539,18 @@ const char *get_filename_ext(const char *filename) {
 	return dot + 1;
 }
 
+/**
+ *
+ * @param filename
+ * @return the type of the file from its filename
+ */
+image_type get_type_from_filename(const gchar *filename) {
+	const char *ext = get_filename_ext(filename);
+	if (!ext)
+		return TYPEUNDEF;
+	return get_type_for_extension(ext);
+}
+
 /**
  * Tests whether the given file is either regular or a symlink
  * @param filename input
@@ -206,6 +571,25 @@ int is_readable_file(const char *filename) {
 	return 0;
 }
 
+static gchar forbidden_char[] = { '/', '\\', '"', '\'' };
+
+gboolean is_forbiden_in_filename(gchar c) {
+	for (int i = 0; i < G_N_ELEMENTS(forbidden_char); i++) {
+		if (c == forbidden_char[i])
+			return TRUE;
+	}
+	return FALSE;
+}
+
+gboolean file_name_has_invalid_chars(const char *name) {
+	if (!name)
+		return TRUE;	// NULL is kind of invalid
+	for (int i = 0; i < strlen(name); i++)
+		if (is_forbiden_in_filename(name[i]))
+			return TRUE;
+	return FALSE;
+}
+
 /** Tests if filename is the canonical name of a known file type
  *  If filename contains an extension, only this file name is tested, else all
  *  extensions are tested for the file name until one is found.
@@ -267,34 +651,6 @@ int stat_file(const char *filename, image_type *type, char **realname) {
 	return 1;
 }
 
-static GUserDirectory sdir[] = { G_USER_DIRECTORY_PICTURES,
-		G_USER_DIRECTORY_DOCUMENTS };
-/** This function tries to set a startup directory. It first looks at the "Pictures" directory,
- *  then if it does not exist, the "Document" one, Finally, if it fails on some UNIX systems
- *  the dir is set to the home directory.
- *  @return a working directory path if success, NULL if error
- */
-gchar *siril_get_startup_dir() {
-	const gchar *dir = NULL;
-	gchar *startup_dir = NULL;
-	gint i = 0;
-	size_t size;
-
-	size = sizeof(sdir) / sizeof(GUserDirectory);
-
-	while (dir == NULL && i < size) {
-		dir = g_get_user_special_dir(sdir[i]);
-		i++;
-	}
-	/* Not every platform has a directory for these logical id */
-	if (dir == NULL) {
-		dir = g_get_home_dir();
-	}
-	if (dir)
-		startup_dir = g_strdup(dir);
-	return startup_dir;
-}
-
 /** Try to change the CWD to the argument, absolute or relative.
  *  If success, the new CWD is written to com.wd
  *  @param[in] dir absolute or relative path we want to set as cwd
@@ -417,14 +773,12 @@ static int ListSequences(const gchar *sDir, const char *sequence_name_to_select,
  */
 int update_sequences_list(const char *sequence_name_to_select) {
 	GtkComboBoxText *seqcombo;
-	struct dirent **list;
 	int number_of_loaded_sequences = 0;
 	int index_of_seq_to_load = -1;
 	char *seqname = NULL;
 
 	// clear the previous list
-	seqcombo = GTK_COMBO_BOX_TEXT(
-			gtk_builder_get_object(builder, "sequence_list_combobox"));
+	seqcombo = GTK_COMBO_BOX_TEXT(lookup_widget("sequence_list_combobox"));
 	gtk_combo_box_text_remove_all(seqcombo);
 
 	if (sequence_name_to_select) {
@@ -439,6 +793,7 @@ int update_sequences_list(const char *sequence_name_to_select) {
 #ifdef _WIN32
 	number_of_loaded_sequences = ListSequences(com.wd, seqname, seqcombo, &index_of_seq_to_load);
 #else
+	struct dirent **list;
 	int i, n;
 
 	n = scandir(com.wd, &list, 0, alphasort);
@@ -471,7 +826,8 @@ int update_sequences_list(const char *sequence_name_to_select) {
 		fprintf(stderr, "No valid sequence found in CWD.\n");
 		return -1;
 	} else {
-		fprintf(stdout, "Loaded %d sequence(s)\n", number_of_loaded_sequences);
+		fprintf(stdout, "Loaded %d %s\n", number_of_loaded_sequences,
+				ngettext("sequence", "sequences", number_of_loaded_sequences));
 	}
 
 	if (number_of_loaded_sequences > 1 && index_of_seq_to_load < 0) {
@@ -505,7 +861,7 @@ void expand_home_in_filename(char *filename, int size) {
 			filename[j + homelen - 1] = filename[j];
 		// the -1 above is tricky: it's the removal of the ~ character from
 		// the original string
-		strncpy(filename, homepath, homelen);
+		memcpy(filename, homepath, homelen);
 	}
 }
 
@@ -515,11 +871,17 @@ void expand_home_in_filename(char *filename, int size) {
  * @param fit input FITS image
  * @return 255 or 65535 if 8- or 16-bit image
  */
-WORD get_normalized_value(fits *fit) {
-	image_find_minmax(fit);
-	if (fit->maxi <= UCHAR_MAX)
-		return UCHAR_MAX;
-	return USHRT_MAX;
+double get_normalized_value(fits *fit) {
+	if (fit->type == DATA_USHORT) {
+		image_find_minmax(fit);
+		if (fit->maxi <= UCHAR_MAX_DOUBLE)
+			return UCHAR_MAX_DOUBLE;
+		return USHRT_MAX_DOUBLE;
+	}
+	if (fit->type == DATA_FLOAT) {
+		return 1.0;
+	}
+	return -1.0;
 }
 
 /**
@@ -554,16 +916,16 @@ char *remove_ext_from_filename(const char *filename) {
  * @param newdata suffix to add
  * @return a new string that should be freed when no longer needed
  */
-char* str_append(char** data, const char* newdata) {
-	char* p;
+gchar* str_append(gchar** data, const gchar* newdata) {
+	gchar* p;
 	int len = (*data ? strlen(*data) : 0);
-	if ((p = realloc(*data, len + strlen(newdata) + 1)) == NULL) {
-		free(p);
+	if ((p = g_try_realloc(*data, len + strlen(newdata) + 1)) == NULL) {
+		g_free(p);
 		PRINT_ALLOC_ERR;
 		return NULL;
 	}
 	*data = p;
-	strcpy(*data + len, newdata);
+	g_strlcpy(*data + len, newdata, len + strlen(newdata));
 	return *data;
 }
 
@@ -595,9 +957,7 @@ char *format_basename(char *root) {
  * @param hi high value
  * @return the computed slope
  */
-float computePente(WORD *lo, WORD *hi) {
-	float pente;
-
+float compute_slope(WORD *lo, WORD *hi) {
 	if (sequence_is_loaded() && !single_image_is_loaded()) {
 		*hi = com.seq.layers[RLAYER].hi;
 		*lo = com.seq.layers[RLAYER].lo;
@@ -606,18 +966,15 @@ float computePente(WORD *lo, WORD *hi) {
 		*hi = com.uniq->layers[RLAYER].hi;
 		*lo = com.uniq->layers[RLAYER].lo;
 	}
-
-	pente = UCHAR_MAX_SINGLE / (float) (*hi - *lo);
-
-	return pente;
+	return UCHAR_MAX_SINGLE / (float) (*hi - *lo);
 }
 
 static const gchar *checking_css_filename() {
 	printf(_("Checking GTK version ... GTK-%d.%d\n"), GTK_MAJOR_VERSION, GTK_MINOR_VERSION);
 	if ((GTK_MAJOR_VERSION >= 3) && (GTK_MINOR_VERSION >= 18))
-		return "gtk.css";
+		return "siril.css";
 	else if ((GTK_MAJOR_VERSION >= 3) && (GTK_MINOR_VERSION < 18))
-		return "gtk_old.css";
+		return "siril_old.css";
 	else {
 		return NULL;
 	}
@@ -639,7 +996,7 @@ void load_css_style_sheet () {
 		exit(1);
 	}
 
-	CSSFile = g_build_filename (com.app_path, css_filename, NULL);
+	CSSFile = g_build_filename(siril_get_system_data_dir(), css_filename, NULL);
 	if (!g_file_test (CSSFile, G_FILE_TEST_EXISTS)) {
 		g_error (_("Unable to load CSS style sheet file: %s. Please reinstall Siril\n"), CSSFile);
 	}
@@ -651,8 +1008,8 @@ void load_css_style_sheet () {
 				GTK_STYLE_PROVIDER(css_provider),
 				GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 		gtk_css_provider_load_from_path(css_provider, CSSFile, NULL);
-		fprintf(stdout, _("Successfully loaded '%s'\n"), CSSFile);
-		g_object_unref (css_provider);
+		g_fprintf(stdout, _("Successfully loaded '%s'\n"), CSSFile);
+		g_object_unref(css_provider);
 	}
 	g_free(CSSFile);
 }
@@ -708,3 +1065,86 @@ double encodeJD(dateTime dt) {
 		return jd1 + 2 - (dt.year / 100) + (dt.year / 400);
 	}
 }
+
+/**
+ * Try to get file info, i.e width and height
+ * @param filename name of the file
+ * @param pixbuf
+ * @return a newly allocated and formated string containing dimension information or NULL
+ */
+gchar* siril_get_file_info(const gchar *filename, GdkPixbuf *pixbuf) {
+	int width, height;
+	int n_channel = 0;
+
+	GdkPixbufFormat *pixbuf_file_info = gdk_pixbuf_get_file_info(filename,
+			&width, &height);
+
+	if (pixbuf) {
+		n_channel = gdk_pixbuf_get_n_channels(pixbuf);
+	}
+
+	if (pixbuf_file_info != NULL) {
+		/* Pixel size of image: width x height in pixel */
+		return g_strdup_printf("%d x %d %s\n%d %s", width, height,
+				ngettext("pixel", "pixels", height), n_channel,
+				ngettext("channel", "channels", n_channel));
+	}
+	return NULL;
+}
+
+/**
+ * Truncate a string str to not exceed an length of size
+ * @param str the string to be truncated
+ * @param size maximum size of the string
+ * @return the truncated size starting by "..." and followed by "/"
+ * if possible
+ */
+gchar *siril_truncate_str(gchar *str, gint size) {
+	GString *trunc_str = g_string_new(str);
+	gint len = strlen(str);
+
+	if (len > size) {
+		gint pos = len - size;
+		/* locate first "/" */
+		char *ptr = strchr(str + pos, G_DIR_SEPARATOR);
+		if (ptr != NULL) {
+			pos = ptr - str;
+		}
+		trunc_str = g_string_erase(trunc_str, 0, pos);
+		trunc_str = g_string_prepend(trunc_str, "...");
+	}
+	return g_string_free(trunc_str, FALSE);
+}
+
+/**
+ * Create a popover with icon and text
+ * @param widget is the parent widget where the popover arises from
+ * @param text will be shown in the popover
+ * @return the GtkWidget of popover
+ */
+GtkWidget* popover_new(GtkWidget *widget, const gchar *text) {
+	GtkWidget *popover, *box, *image, *label;
+
+	popover = gtk_popover_new(widget);
+	label = gtk_label_new(NULL);
+	box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
+	image = gtk_image_new_from_icon_name("dialog-information-symbolic",
+			GTK_ICON_SIZE_DIALOG);
+
+	gtk_label_set_markup(GTK_LABEL(label), text);
+	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+	gtk_label_set_max_width_chars(GTK_LABEL(label), 64);
+
+	gtk_box_pack_start(GTK_BOX(box), image, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
+	gtk_container_add(GTK_CONTAINER(popover), box);
+
+	/* make all sensitive in case where parent is not */
+	gtk_widget_set_sensitive(label, TRUE);
+	gtk_widget_set_sensitive(box, TRUE);
+	gtk_widget_set_sensitive(popover, TRUE);
+
+	gtk_widget_show_all(box);
+
+	return popover;
+}
