@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2019 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -19,23 +19,28 @@
  */
 
 #include <gsl/gsl_histogram.h>
-#include <assert.h>
 #include <string.h>
 #include <stdint.h>
 #include <math.h>
 #include <float.h>
 #include "core/siril.h"
 #include "core/proto.h"
+#include "core/processing.h"
+#include "core/siril_app_dirs.h"
 #include "algos/statistics.h"
 #include "io/single_image.h"
-#include "gui/histogram.h"
+#include "io/sequence.h"
+#include "gui/image_display.h"
 #include "gui/callbacks.h"	// for lookup_widget()
 #include "gui/progress_and_log.h"
 #include "gui/dialogs.h"
+#include "gui/siril_preview.h"
 #include "core/undo.h"
 
-#define shadowsClipping -2.80 /* Shadows clipping point measured in sigma units from the main histogram peak. */
-#define targetBackground 0.25 /* final "luminance" of the image for autostretch in the [0,1] range */
+#include "histogram.h"
+
+#define shadowsClipping -2.80f /* Shadows clipping point measured in sigma units from the main histogram peak. */
+#define targetBackground 0.25f /* final "luminance" of the image for autostretch in the [0,1] range */
 #define GRADIENT_HEIGHT 12
 
 #undef HISTO_DEBUG
@@ -51,17 +56,16 @@
 static double histo_color_r[] = { 1.0, 0.0, 0.0, 0.0 };
 static double histo_color_g[] = { 0.0, 1.0, 0.0, 0.0 };
 static double histo_color_b[] = { 0.0, 0.0, 1.0, 0.0 };
-static double graph_height = 0.;	// the max value of all bins
+static float graph_height = 0.f;	// the max value of all bins
 static uint64_t clipped[] = { 0, 0 };
 
 static GtkToggleToolButton *toggles[MAXVPORT] = { NULL };
 static GtkToggleToolButton *toggleGrid = NULL, *toggleCurve = NULL;
 
-/* the original image and its histogram, used as starting point of each computation */
-static fits histo_gfit_backup;
+/* the original histogram, used as starting point of each computation */
 static gsl_histogram *hist_backup[MAXVPORT] = { NULL, NULL, NULL, NULL };
 
-static double _midtones, _shadows, _highlights;
+static float _midtones, _shadows, _highlights;
 
 static gboolean _click_on_histo = FALSE;
 static ScaleType _type_of_scale;
@@ -79,8 +83,7 @@ static int get_height_of_histo() {
 
 static void clear_hist_backup() {
 	if (hist_backup[0]) {
-		int i;
-		for (i = 0; i < gfit.naxes[2]; i++) {
+		for (int i = 0; i < gfit.naxes[2]; i++) {
 			gsl_histogram_free(hist_backup[i]);
 			hist_backup[i] = NULL;
 		}
@@ -88,11 +91,10 @@ static void clear_hist_backup() {
 }
 
 static void histo_startup() {
-	copyfits(&gfit, &histo_gfit_backup, CP_ALLOC | CP_COPYA | CP_FORMAT, -1);
+	copy_gfit_to_backup();
 	// also get the backup histogram
 	compute_histo_for_gfit();
-	int i;
-	for (i = 0; i < gfit.naxes[2]; i++)
+	for (int i = 0; i < gfit.naxes[2]; i++)
 		hist_backup[i] = gsl_histogram_clone(com.layers_hist[i]);
 }
 
@@ -105,7 +107,7 @@ static void histo_close(gboolean revert) {
 			set_histogram(hist_backup[i], i);
 			hist_backup[i] = NULL;
 		}
-		copyfits(&histo_gfit_backup, &gfit, CP_COPYA, -1);
+		copy_backup_to_gfit();
 		adjust_cutoff_from_updated_gfit();
 		redraw(com.cvport, REMAP_ALL);
 		redraw_previews();
@@ -113,15 +115,15 @@ static void histo_close(gboolean revert) {
 	}
 
 	// free data
-	clearfits(&histo_gfit_backup);
+	clear_backup();
 	clear_hist_backup();
 }
 
 static void histo_recompute() {
 	set_cursor("progress");
-	copyfits(&histo_gfit_backup, &gfit, CP_COPYA, -1);
+	copy_backup_to_gfit();
 
-	apply_mtf_to_fits(&histo_gfit_backup, &gfit);
+	apply_mtf_to_fits(get_preview_gfit_backup(), &gfit);
 	// com.layers_hist should be good, update_histo_mtf() is always called before
 
 	adjust_cutoff_from_updated_gfit();
@@ -151,18 +153,18 @@ static void _update_entry_text() {
 	GtkEntry *histoHighEntry = GTK_ENTRY(lookup_widget("histoHighEntry"));
 	gchar *buffer;
 
-	buffer = g_strdup_printf("%.7lf", _shadows);
+	buffer = g_strdup_printf("%.7f", _shadows);
 	gtk_entry_set_text(histoShadEntry, buffer);
 	g_free(buffer);
-	buffer = g_strdup_printf("%.7lf", _highlights);
+	buffer = g_strdup_printf("%.7f", _highlights);
 	gtk_entry_set_text(histoHighEntry, buffer);
 	g_free(buffer);
-	buffer = g_strdup_printf("%.7lf", _midtones);
+	buffer = g_strdup_printf("%.7f", _midtones);
 	gtk_entry_set_text(histoMidEntry, buffer);
 	g_free(buffer);
 }
 
-static void _update_clipped_pixels(int data) {
+static void _update_clipped_pixels(size_t data) {
 	static GtkEntry *clip_high = NULL, *clip_low = NULL;
 	double tmp;
 	char buffer[16];
@@ -171,17 +173,16 @@ static void _update_clipped_pixels(int data) {
 		clip_high = GTK_ENTRY(lookup_widget("clip_highlights"));
 		clip_low = GTK_ENTRY(lookup_widget("clip_shadows"));
 	}
-	tmp = (double)clipped[1] * 100.0 / data;
-	g_snprintf(buffer, sizeof(buffer), "%.3lf%%", tmp);
+	tmp = (double)clipped[1] * 100.0 / (double)data;
+	g_snprintf(buffer, sizeof(buffer), "%.3f%%", tmp);
 	gtk_entry_set_text(clip_high, buffer);
-	tmp = (double)clipped[0] * 100.0 / data;
-	g_snprintf(buffer, sizeof(buffer), "%.3lf%%", tmp);
+	tmp = (double)clipped[0] * 100.0 / (double)data;
+	g_snprintf(buffer, sizeof(buffer), "%.3f%%", tmp);
 	gtk_entry_set_text(clip_low, buffer);
-
 }
 
 static int is_histogram_visible() {
-	GtkWidget *window = lookup_widget("histogram_window");
+	GtkWidget *window = lookup_widget("histogram_dialog");
 	return gtk_widget_get_visible(window);
 }
 
@@ -206,11 +207,11 @@ static void set_histo_toggles_names() {
 	if (gfit.naxis == 2) {
 		gtk_widget_set_tooltip_text(GTK_WIDGET(toggles[0]), _("Gray channel"));
 		GtkWidget *w;
-		if (com.want_dark) {
-			image = g_build_filename(com.app_path, "pixmaps", "monochrome_dark.png", NULL);
+		if (com.pref.combo_theme == 0) {
+			image = g_build_filename(siril_get_system_data_dir(), "pixmaps", "monochrome_dark.png", NULL);
 			w = gtk_image_new_from_file(image);
 		} else {
-			image = g_build_filename(com.app_path, "pixmaps", "monochrome.png", NULL);
+			image = g_build_filename(siril_get_system_data_dir(), "pixmaps", "monochrome.png", NULL);
 			w = gtk_image_new_from_file(image);
 		}
 		gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(toggles[0]), w);
@@ -227,7 +228,7 @@ static void set_histo_toggles_names() {
 			gtk_widget_set_visible(GTK_WIDGET(toggles[3]), FALSE);
 
 	} else {
-		image = g_build_filename(com.app_path, "pixmaps", "r.png", NULL);
+		image = g_build_filename(siril_get_system_data_dir(), "pixmaps", "r.png", NULL);
 		gtk_widget_set_tooltip_text(GTK_WIDGET(toggles[0]), _("Red channel"));
 		GtkWidget *w = gtk_image_new_from_file(image);
 		gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(toggles[0]), w);
@@ -282,22 +283,55 @@ static void adjust_histogram_vport_size() {
 #endif
 }
 
-// create a new histrogram object for the passed fit and layer
-gsl_histogram* computeHisto(fits* fit, int layer) {
-	assert(layer < 3);
+size_t get_histo_size(fits *fit) {
+	if (fit->type == DATA_USHORT)
+		return (size_t)get_normalized_value(fit);
+	return (size_t)USHRT_MAX;
+}
+
+// create a new hitogram object for the passed fit and layer
+gsl_histogram* computeHisto(fits *fit, int layer) {
+	g_assert(layer < 3);
 	size_t i, ndata, size;
-	WORD *buf;
 
-	size = (size_t)get_normalized_value(fit);
-	gsl_histogram* histo = gsl_histogram_alloc(size + 1);
-	gsl_histogram_set_ranges_uniform(histo, 0, size);
+	size = get_histo_size(fit);
+	gsl_histogram *histo = gsl_histogram_alloc(size + 1);
+	gsl_histogram_set_ranges_uniform(histo, 0, fit->type == DATA_FLOAT ? 1.0 + 1.0 / size : size + 1);
+	ndata = fit->naxes[0] * fit->naxes[1];
 
-	buf = fit->pdata[layer];
-	ndata = fit->rx * fit->ry;
-//#pragma omp parallel for num_threads(com.max_thread) private(i) schedule(static) // cause errors !!!
-	for (i = 0; i < ndata; i++) {
-		gsl_histogram_increment(histo, (double)buf[i]);
+#ifdef _OPENMP
+#pragma omp parallel num_threads(com.max_thread)
+#endif
+	{
+		gsl_histogram *histo_thr = gsl_histogram_alloc(size + 1);
+		gsl_histogram_set_ranges_uniform(histo_thr, 0, fit->type == DATA_FLOAT ? 1.0 + 1.0 / size : size + 1);
+
+		if (fit->type == DATA_USHORT) {
+			WORD *buf = fit->pdata[layer];
+#ifdef _OPENMP
+#pragma omp for private(i) schedule(static)
+#endif
+			for (i = 0; i < ndata; i++) {
+				gsl_histogram_increment(histo_thr, (double) buf[i]);
+			}
+		} else if (fit->type == DATA_FLOAT) {
+			float *buf = fit->fpdata[layer];
+#ifdef _OPENMP
+#pragma omp for private(i) schedule(static)
+#endif
+			for (i = 0; i < ndata; i++) {
+				gsl_histogram_increment(histo_thr, (double) buf[i]);
+			}
+		}
+#ifdef _OPENMP
+#pragma omp critical
+#endif
+		{
+			gsl_histogram_add(histo, histo_thr);
+		}
+		gsl_histogram_free(histo_thr);
 	}
+
 	return histo;
 }
 
@@ -308,10 +342,10 @@ static void draw_curve(cairo_t *cr, int width, int height) {
 	cairo_set_line_width(cr, 1.0);
 	cairo_set_source_rgb(cr, .9, .9, .9);
 
-	for (k = 0; k < USHRT_MAX + 1; k++) {
-		double x = k / (USHRT_MAX_SINGLE);
-		double y = MTF(x, _midtones, _shadows, _highlights);
-		cairo_line_to(cr, x * width, height * (1 - y));
+	for (k = 0; k < width + 1; k++) {
+		float x = k / (float) width;
+		float y = MTF(x, _midtones, _shadows, _highlights);
+		cairo_line_to(cr, k, height * (1 - y));
 	}
 	cairo_stroke(cr);
 }
@@ -419,7 +453,7 @@ static void draw_slider(cairo_t *cr, int width, int height, int xpos) {
 
 static void display_scale(cairo_t *cr, int width, int height) {
 	draw_gradient(cr, width, height);
-	double delta = ((_highlights - _shadows) * _midtones) + _shadows;
+	float delta = ((_highlights - _shadows) * _midtones) + _shadows;
 	draw_slider(cr, width, height, _shadows * width);
 	draw_slider(cr, width, height, (delta) * width);
 	draw_slider(cr, width, height, _highlights * width);
@@ -427,24 +461,25 @@ static void display_scale(cairo_t *cr, int width, int height) {
 
 static void display_histo(gsl_histogram *histo, cairo_t *cr, int layer, int width,
 		int height, double zoomH, double zoomV) {
+	if (width <= 0) return;
 	int current_bin;
 	size_t norm = gsl_histogram_bins(histo) - 1;
 
-	float vals_per_px = (float)(norm) / (float)width;	// size of a bin
+	float vals_per_px = (float)norm / (float)width;	// size of a bin
 	size_t i, nb_orig_bins = gsl_histogram_bins(histo);
 
 	// We need to store the binned histogram in order to find the binned maximum
-	static double *displayed_values = NULL;
+	static gfloat *displayed_values = NULL;
 	static int nb_bins_allocated = 0;
 	/* we create a bin for each pixel in the displayed width.
 	 * nb_bins_allocated is thus equal to the width of the image */
 	if (nb_bins_allocated != width) {
-		double *tmp;
+		gfloat *tmp;
 		nb_bins_allocated = width;
-		tmp = realloc(displayed_values, nb_bins_allocated * sizeof(double));
+		tmp = realloc(displayed_values, nb_bins_allocated * sizeof(gfloat));
 		if (!tmp) {
-			if (displayed_values) {
-				free(displayed_values);
+			if (displayed_values != NULL) {
+				g_free(displayed_values);
 				displayed_values = NULL;
 			}
 			PRINT_ALLOC_ERR;
@@ -454,7 +489,6 @@ static void display_histo(gsl_histogram *histo, cairo_t *cr, int layer, int widt
 		displayed_values = tmp;
 		memset(displayed_values, 0, nb_bins_allocated);
 	}
-	assert(displayed_values);
 
 	if (gfit.naxis == 2)
 		cairo_set_source_rgb(cr, 255.0, 255.0, 255.0);
@@ -468,14 +502,14 @@ static void display_histo(gsl_histogram *histo, cairo_t *cr, int layer, int widt
 	i = 0;
 	current_bin = 0;
 	do {
-		double bin_val = 0.0;
+		float bin_val = 0.f;
 		while (i < nb_orig_bins
 				&& (float)i / vals_per_px <= (float)current_bin + 0.5f) {
-			bin_val += gsl_histogram_get(histo, i);
+			bin_val += (float)gsl_histogram_get(histo, i);
 			i++;
 		}
-		if (is_log_scale() && bin_val != 0.0) {
-			bin_val = log(bin_val);
+		if (is_log_scale() && bin_val != 0.f) {
+			bin_val = logf(bin_val);
 		}
 		displayed_values[current_bin] = bin_val;
 		if (bin_val > graph_height)	// check for maximum
@@ -483,47 +517,55 @@ static void display_histo(gsl_histogram *histo, cairo_t *cr, int layer, int widt
 		current_bin++;
 	} while (i < nb_orig_bins && current_bin < nb_bins_allocated);
 	for (i = 0; i < nb_bins_allocated; i++) {
-		double bin_height = height - height * displayed_values[i] / graph_height;
+		float bin_height = height - height * displayed_values[i] / graph_height;
 		cairo_line_to(cr, i, bin_height);
 	}
 	cairo_stroke(cr);
 }
 
 static void apply_mtf_to_fits(fits *from, fits *to) {
-	int i, chan, nb_chan, ndata;
-	double norm = (double)get_normalized_value(from);
 
-	assert(from->naxes[2] == 1 || from->naxes[2] == 3);
-	nb_chan = from->naxes[2];
-	ndata = from->rx * from->ry;
+	g_assert(from->naxes[2] == 1 || from->naxes[2] == 3);
+	const size_t ndata = from->naxes[0] * from->naxes[1] * from->naxes[2];
+	g_assert(from->type == to->type);
 
-	for (chan = 0; chan < nb_chan; chan++) {
+	if (from->type == DATA_USHORT) {
+		float norm = (float)get_normalized_value(from);
 #ifdef _OPENMP
-#pragma omp parallel for num_threads(com.max_thread) private(i) schedule(static)
+#pragma omp parallel for num_threads(com.max_thread) schedule(static)
 #endif
-		for (i = 0; i < ndata; i++) {
-			double pxl = ((double)from->pdata[chan][i] / norm);
-			double mtf = MTF(pxl, _midtones, _shadows, _highlights);
-			to->pdata[chan][i] = round_to_WORD(mtf * norm);
+		for (size_t i = 0; i < ndata; i++) {
+			float pxl = (float)from->data[i] / norm;
+			float mtf = MTF(pxl, _midtones, _shadows, _highlights);
+			to->data[i] = round_to_WORD(mtf * norm);
 		}
 	}
+	else if (from->type == DATA_FLOAT) {
+#ifdef _OPENMP
+#pragma omp parallel for num_threads(com.max_thread) schedule(static)
+#endif
+		for (size_t i = 0; i < ndata; i++) {
+			to->fdata[i] = MTF(from->fdata[i], _midtones, _shadows, _highlights);
+		}
+	}
+	else return;
 
 	invalidate_stats_from_fit(to);
 }
 
-static void apply_mtf_to_histo(gsl_histogram *histo, double norm,
-		double m, double lo, double hi) {
-	gsl_histogram *mtf_histo;
-	unsigned short i;
+static void apply_mtf_to_histo(gsl_histogram *histo, float norm,
+		float m, float lo, float hi) {
+
+	size_t int_norm = (size_t)norm;
+	gsl_histogram *mtf_histo = gsl_histogram_alloc(int_norm + 1);
 
-	mtf_histo = gsl_histogram_alloc((size_t)norm + 1);
 	gsl_histogram_set_ranges_uniform(mtf_histo, 0, norm);
 
 // #pragma omp parallel for num_threads(com.max_thread) private(i) schedule(static) // disabled because of ISSUE #136 (https://free-astro.org/bugs/view.php?id=136)
-	for (i = 0; i < round_to_WORD(norm); i++) {
+	for (size_t i = 0; i < int_norm + 1; i++) {
 		WORD mtf;
-		double binval = gsl_histogram_get(histo, i);
-		double pxl = ((double)i / norm);
+		float binval = gsl_histogram_get(histo, i);
+		float pxl = ((float)i / norm);
 		uint64_t clip[2] = { 0, 0 };
 
 		if (i < round_to_WORD(lo * norm)) {
@@ -534,7 +576,7 @@ static void apply_mtf_to_histo(gsl_histogram *histo, double norm,
 			clip[1] += binval;
 		}
 		mtf = round_to_WORD(MTF(pxl, m, lo, hi) * norm);
-		gsl_histogram_accumulate(mtf_histo, mtf, binval);
+		gsl_histogram_accumulate(mtf_histo, (double)mtf, (double)binval);
 //#ifdef _OPENMP
 //#pragma omp critical
 //#endif
@@ -548,10 +590,10 @@ static void apply_mtf_to_histo(gsl_histogram *histo, double norm,
 }
 
 static void reset_cursors_and_values() {
-	_shadows = 0.0;
-	_midtones = 0.5;
-	_highlights = 1.0;
-	graph_height = 0.;
+	_shadows = 0.f;
+	_midtones = 0.5f;
+	_highlights = 1.0f;
+	graph_height = 0.f;
 
 	_init_clipped_pixels();
 	_initialize_clip_text();
@@ -567,21 +609,20 @@ static void queue_window_redraw() {
 }
 
 static void update_histo_mtf() {
-	unsigned int i, data = 0;
-	double norm = (double)gsl_histogram_bins(com.layers_hist[0]) - 1;
+	float norm = (float)gsl_histogram_bins(com.layers_hist[0]) - 1;
 
 	_init_clipped_pixels();
-	for (i = 0; i < gfit.naxes[2]; i++) {
+	for (long i = 0; i < gfit.naxes[2]; i++) {
 		gsl_histogram_memcpy(com.layers_hist[i], hist_backup[i]);
 		apply_mtf_to_histo(com.layers_hist[i], norm, _midtones, _shadows, _highlights);
 	}
-	data = gfit.rx * gfit.ry * gfit.naxes[2];
+	size_t data = gfit.naxes[0] * gfit.naxes[1] * gfit.naxes[2];
 	_update_clipped_pixels(data);
 	queue_window_redraw();
 }
 
 static void set_histogram(gsl_histogram *histo, int layer) {
-	assert(layer >= 0 && layer < MAXVPORT);
+	g_assert(layer >= 0 && layer < MAXVPORT);
 	if (com.layers_hist[layer])
 		gsl_histogram_free(com.layers_hist[layer]);
 	com.layers_hist[layer] = histo;
@@ -598,35 +639,52 @@ static gboolean on_gradient(GdkEvent *event, int width, int height) {
  * Public functions
  */
 
+void mtf_with_parameters(fits *fit, float lo, float mid, float hi) {
+	_shadows = lo;
+	_midtones = mid;
+	_highlights = hi;
+	apply_mtf_to_fits(fit, fit);
+}
+
 gsl_histogram* computeHisto_Selection(fits* fit, int layer,
 		rectangle *selection) {
-	assert(layer < 3);
-	WORD *from;
-	size_t stridefrom, i, j, size;
+	g_assert(layer < 3);
 
-	size = (size_t)get_normalized_value(fit);
+	size_t size = get_histo_size(fit);
 	gsl_histogram* histo = gsl_histogram_alloc(size + 1);
-	gsl_histogram_set_ranges_uniform(histo, 0, size);
+	gsl_histogram_set_ranges_uniform(histo, 0, fit->type == DATA_FLOAT ? 1.0 : size);
+	size_t stridefrom = fit->rx - selection->w;
 
-	from = fit->pdata[layer] + (fit->ry - selection->y - selection->h) * fit->rx
+	if (fit->type == DATA_USHORT) {
+		WORD *from = fit->pdata[layer] + (fit->ry - selection->y - selection->h) * fit->rx
 			+ selection->x;
-	stridefrom = fit->rx - selection->w;
-	for (i = 0; i < selection->h; i++) {
-		for (j = 0; j < selection->w; j++) {
-			gsl_histogram_increment(histo, (double) *from);
-			from++;
+		for (size_t i = 0; i < selection->h; i++) {
+			for (size_t j = 0; j < selection->w; j++) {
+				gsl_histogram_increment(histo, (double)*from);
+				from++;
+			}
+			from += stridefrom;
+		}
+	}
+	else if (fit->type == DATA_FLOAT) {
+		float *from = fit->fpdata[layer] + (fit->ry - selection->y - selection->h) * fit->rx
+			+ selection->x;
+		for (size_t i = 0; i < selection->h; i++) {
+			for (size_t j = 0; j < selection->w; j++) {
+				gsl_histogram_increment(histo, (double)*from);
+				from++;
+			}
+			from += stridefrom;
 		}
-		from += stridefrom;
 	}
 	return histo;
 }
 
 void compute_histo_for_gfit() {
 	int nb_layers = 3;
-	int i;
 	if (gfit.naxis == 2)
 		nb_layers = 1;
-	for (i = 0; i < nb_layers; i++) {
+	for (int i = 0; i < nb_layers; i++) {
 		if (!com.layers_hist[i])
 			set_histogram(computeHisto(&gfit, i), i);
 	}
@@ -634,8 +692,7 @@ void compute_histo_for_gfit() {
 }
 
 void invalidate_gfit_histogram() {
-	int layer;
-	for (layer = 0; layer < MAXVPORT; layer++) {
+	for (int layer = 0; layer < MAXVPORT; layer++) {
 		set_histogram(NULL, layer);
 	}
 }
@@ -648,37 +705,36 @@ void update_gfit_histogram_if_needed() {
 }
 
 void clear_histograms() {
-	int i;
-	for (i = 0; i < MAXVPORT; i++) {
+	for (int i = 0; i < MAXVPORT; i++) {
 		set_histogram(NULL, i);
 	}
 	// TODO: call histo_close? should it be done by the caller?
 }
 
-double MTF(double x, double m, double lo, double hi) {
+float MTF(float x, float m, float lo, float hi) {
 	if (x <= lo)
-		return 0.0;
+		return 0.f;
 	if (x >= hi)
-		return 1.0;
+		return 1.f;
 
-	double xp = (x - lo) / (hi - lo);
+	float xp = (x - lo) / (hi - lo);
 
-	return ((m - 1.0) * xp) / (((2.0 * m - 1.0) * xp) - m);
+	return ((m - 1.f) * xp) / (((2.f * m - 1.f) * xp) - m);
 }
 
-double findMidtonesBalance(fits *fit, double *shadows, double *highlights) {
-	double c0 = 0.0, c1 = 0.0;
-	double m = 0.0;
+float findMidtonesBalance(fits *fit, float *shadows, float *highlights) {
+	float c0 = 0.0, c1 = 0.0;
+	float m = 0.0;
 	int i, n, invertedChannels = 0;
 	imstats *stat[3];
 
 	n = fit->naxes[2];
 
 	for (i = 0; i < n; ++i) {
-		stat[i] = statistics(NULL, -1, fit, i, NULL, STATS_BASIC | STATS_MAD);
+		stat[i] = statistics(NULL, -1, fit, i, NULL, STATS_BASIC | STATS_MAD, TRUE);
 		if (!stat[i]) {
 			siril_log_message(_("Error: statistics computation failed.\n"));
-			return 0.0;
+			return 0.f;
 		}
 
 		if (stat[i]->median / stat[i]->normValue > 0.5)
@@ -687,41 +743,41 @@ double findMidtonesBalance(fits *fit, double *shadows, double *highlights) {
 
 	if (invertedChannels < n) {
 		for (i = 0; i < n; ++i) {
-			double median, mad, normValue;
+			float median, mad, normValue;
 
-			normValue = stat[i]->normValue;
-			median = stat[i]->median / normValue;
-			mad = stat[i]->mad / normValue * MAD_NORM;
+			normValue = (float)stat[i]->normValue;
+			median = (float) stat[i]->median / normValue;
+			mad = (float) stat[i]->mad / normValue * (float)MAD_NORM;
 			/* this is a guard to avoid breakdown point */
-			if (mad == 0.0) mad = 0.001;
+			if (mad == 0.f) mad = 0.001f;
 
 			c0 += median + shadowsClipping * mad;
 			m += median;
 		}
-		c0 /= (double) n;
-		if (c0 < 0.0) c0 = 0.0;
-		double m2 = m / (double) n - c0;
-		m = MTF(m2, targetBackground, 0.0, 1.0);
+		c0 /= (float) n;
+		if (c0 < 0.f) c0 = 0.f;
+		float m2 = m / (float) n - c0;
+		m = MTF(m2, targetBackground, 0.f, 1.f);
 		*shadows = c0;
 		*highlights = 1.0;
 	} else {
 		for (i = 0; i < n; ++i) {
-			double median, mad, normValue;
+			float median, mad, normValue;
 
-			normValue = stat[i]->normValue;
-			median = stat[i]->median / normValue;
-			mad = stat[i]->mad / normValue * MAD_NORM;
+			normValue = (float) stat[i]->normValue;
+			median = (float) stat[i]->median / normValue;
+			mad = (float) stat[i]->mad / normValue * (float)MAD_NORM;
 			/* this is a guard to avoid breakdown point */
-			if (mad == 0.0) mad = 0.001;
+			if (mad == 0.f) mad = 0.001f;
 
 			m += median;
 			c1 += median - shadowsClipping * mad;
 		}
-		c1 /= (double) n;
-		if (c1 > 1.0) c1 = 1.0;
-		double m2 = c1 - m / (double) n;
-		m = 1.0 - MTF(m2, targetBackground, 0.0, 1.0);
-		*shadows = 0.0;
+		c1 /= (float) n;
+		if (c1 > 1.f) c1 = 1.f;
+		float m2 = c1 - m / (float) n;
+		m = 1.f - MTF(m2, targetBackground, 0.f, 1.f);
+		*shadows = 0.f;
 		*highlights = c1;
 
 	}
@@ -730,6 +786,14 @@ double findMidtonesBalance(fits *fit, double *shadows, double *highlights) {
 	return m;
 }
 
+static int mtf_image_hook(struct generic_seq_args *args, int o, int i, fits *fit,
+		rectangle *_) {
+	struct mtf_data *m_args = (struct mtf_data*) args->user;
+
+	mtf_with_parameters(fit, m_args->lo, m_args->mid, m_args->hi);
+	return 0;
+}
+
 /* Callback functions */
 
 gboolean redraw_histo(GtkWidget *widget, cairo_t *cr, gpointer data) {
@@ -775,7 +839,7 @@ void on_button_histo_close_clicked(GtkButton *button, gpointer user_data) {
 	reset_cursors_and_values();
 	histo_close(TRUE);
 	set_cursor_waiting(FALSE);
-	siril_close_dialog("histogram_window");
+	siril_close_dialog("histogram_dialog");
 }
 
 void on_button_histo_reset_clicked(GtkButton *button, gpointer user_data) {
@@ -795,24 +859,51 @@ gboolean on_scale_key_release_event(GtkWidget *widget, GdkEvent *event,
 }
 
 void on_button_histo_apply_clicked(GtkButton *button, gpointer user_data) {
-	if ((_midtones != 0.5) || (_shadows != 0.0) || (_highlights != 1.0)) {
+	if ((_midtones == 0.5f) && (_shadows == 0.f) && (_highlights == 1.f)) {
+		return;
+	}
+
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("checkMTFSeq")))
+			&& sequence_is_loaded()) {
+		/* Apply to the whole sequence */
+		struct mtf_data *args = malloc(sizeof(struct mtf_data));
+
+		args->lo = _shadows;
+		args->mid = _midtones;
+		args->hi = _highlights;
+		args->seqEntry = gtk_entry_get_text(GTK_ENTRY(lookup_widget("entryMTFSeq")));
+		if (args->seqEntry && args->seqEntry[0] == '\0')
+			args->seqEntry = "mtf_";
+		args->seq = &com.seq;
+		/* here it is a bit tricky.
+		 * It is better to first close the window as it is a liveview tool
+		 * TODO: could we improve this behavior?
+		 */
+		reset_cursors_and_values();
+		histo_close(TRUE);
+		siril_close_dialog("histogram_dialog");
+
+		/* apply the process */
+		apply_mtf_to_sequence(args);
+
+	} else {
 		// the apply button resets everything after recomputing with the current values
 		histo_recompute();
 		// partial cleanup
-		fprintf(stdout, "Applying histogram (mid=%.3lf, lo=%.3lf, hi=%.3lf)\n",
+		siril_debug_print("Applying histogram (mid=%.3f, lo=%.3f, hi=%.3f)\n",
 				_midtones, _shadows, _highlights);
-		undo_save_state(&histo_gfit_backup, "Processing: Histogram Transf. "
-				"(mid=%.3lf, lo=%.3lf, hi=%.3lf)", _midtones, _shadows,
+		undo_save_state(get_preview_gfit_backup(),
+				_("Histogram Transf. (mid=%.3f, lo=%.3f, hi=%.3f)"), _midtones, _shadows,
 				_highlights);
 
-		clearfits(&histo_gfit_backup);
+		clear_backup();
 		clear_hist_backup();
 		// reinit
 		histo_startup();
 		reset_cursors_and_values();
-	}
 
-	set_cursor("default");
+		set_cursor("default");
+	}
 }
 
 void apply_histo_cancel() {
@@ -838,14 +929,14 @@ void on_histoSpinZoom_value_changed(GtkRange *range, gpointer user_data) {
 }
 
 void on_histoToolAutoStretch_clicked(GtkToolButton *button, gpointer user_data) {
-	double m, shadows = 0.0, highlights = 0.0;
+	float m, shadows = 0.f, highlights = 0.f;
 
 	set_cursor_waiting(TRUE);
 	/* we always apply this function on original data */
-	m = findMidtonesBalance(&histo_gfit_backup, &shadows, &highlights);
+	m = findMidtonesBalance(get_preview_gfit_backup(), &shadows, &highlights);
 	_shadows = shadows;
 	_midtones = m;
-	_highlights = 1.0;
+	_highlights = 1.f;
 
 	_update_entry_text();
 	update_histo_mtf();
@@ -856,14 +947,18 @@ void on_histoToolAutoStretch_clicked(GtkToolButton *button, gpointer user_data)
 void on_menuitem_histo_activate(GtkMenuItem *menuitem, gpointer user_data) {
 	set_cursor_waiting(TRUE);
 
-	siril_open_dialog("histogram_window");
+	siril_open_dialog("histogram_dialog");
 	set_cursor_waiting(FALSE);
 }
 
 void toggle_histogram_window_visibility(GtkToolButton *button, gpointer user_data) {
-	if (gtk_widget_get_visible(lookup_widget("histogram_window")))
-		siril_close_dialog("histogram_window");
-	else {
+	if (gtk_widget_get_visible((GtkWidget *)user_data)) {
+		set_cursor_waiting(TRUE);
+		reset_cursors_and_values();
+		histo_close(TRUE);
+		set_cursor_waiting(FALSE);
+		siril_close_dialog("histogram_dialog");
+	} else {
 		on_menuitem_histo_activate(NULL, NULL);
 	}
 }
@@ -894,12 +989,12 @@ gboolean on_drawingarea_histograms_motion_notify_event(GtkWidget *widget, GdkEve
 
 		switch (_type_of_scale) {
 		case SCALE_LOW:
-			if (xpos > _highlights) {
+			if ((float)xpos > _highlights) {
 				_shadows = _highlights;
 			} else {
-				_shadows = xpos;
+				_shadows = (float)xpos;
 			}
-			buffer = g_strdup_printf("%.7lf", _shadows);
+			buffer = g_strdup_printf("%.7f", _shadows);
 			gtk_entry_set_text(histoShadEntry, buffer);
 			break;
 
@@ -907,21 +1002,21 @@ gboolean on_drawingarea_histograms_motion_notify_event(GtkWidget *widget, GdkEve
 			if (_highlights == _shadows) {
 				_midtones = _highlights;
 			} else {
-				_midtones = (xpos - _shadows) / (_highlights - _shadows);
+				_midtones = ((float)xpos - _shadows) / (_highlights - _shadows);
 			}
-			if (_midtones > 1.0) _midtones = 1.0;
-			if (_midtones < 0.0) _midtones = 0.0;
-			buffer = g_strdup_printf("%.7lf", _midtones);
+			if (_midtones > 1.f) _midtones = 1.f;
+			if (_midtones < 0.f) _midtones = 0.f;
+			buffer = g_strdup_printf("%.7f", _midtones);
 			gtk_entry_set_text(histoMidEntry, buffer);
 			break;
 
 		case SCALE_HI:
-			if (xpos < _shadows) {
+			if ((float)xpos < _shadows) {
 				_shadows =_highlights;
 			} else {
-				_highlights = xpos;
+				_highlights = (float)xpos;
 			}
-			buffer = g_strdup_printf("%.7lf", _highlights);
+			buffer = g_strdup_printf("%.7f", _highlights);
 			gtk_entry_set_text(histoHighEntry, buffer);
 			break;
 		}
@@ -943,17 +1038,17 @@ gboolean on_drawingarea_histograms_button_press_event(GtkWidget *widget,
 	int height = get_height_of_histo();
 
 	if (on_gradient((GdkEvent *) event, width, height)) {
-		double delta = ((_highlights - _shadows) * _midtones) + _shadows;
+		float delta = ((_highlights - _shadows) * _midtones) + _shadows;
 
 		_click_on_histo = TRUE;
 		gdouble xpos = ((GdkEventButton*) event)->x / (gdouble) width;
-		if (fabs(xpos - _highlights) < fabs(xpos - _shadows) && fabs(xpos - _highlights) < fabs(xpos - delta)) {
+		if (fabsf((float)xpos - _highlights) < fabsf((float)xpos - _shadows) && fabsf((float)xpos - _highlights) < fabsf((float)xpos - delta)) {
 			_type_of_scale = SCALE_HI;
-		} else if (fabs(xpos - _shadows) < fabs(xpos - delta) && fabs(xpos - _shadows) < fabs(xpos - _highlights)) {
+		} else if (fabsf((float)xpos - _shadows) < fabsf((float)xpos - delta) && fabsf((float)xpos - _shadows) < fabsf((float)xpos - _highlights)) {
 			_type_of_scale = SCALE_LOW;
-		} else if ((_shadows == _highlights) && (_shadows > 0.0)) {
+		} else if ((_shadows == _highlights) && (_shadows > 0.f)) {
 			_type_of_scale = SCALE_LOW;
-		} else if ((_shadows == _highlights) && (_shadows == 0.0)) {
+		} else if ((_shadows == _highlights) && (_shadows == 0.f)) {
 			_type_of_scale = SCALE_HI;
 		} else {
 			_type_of_scale = SCALE_MID;
@@ -978,7 +1073,7 @@ gboolean on_drawingarea_histograms_button_release_event(GtkWidget *widget,
 }
 
 void on_histoMidEntry_activate(GtkEntry *entry, gpointer user_data) {
-	double mid = atof(gtk_entry_get_text(entry));
+	float mid = atof(gtk_entry_get_text(entry));
 	if (mid <= _shadows) mid = _shadows;
 	if (mid >= _highlights) mid = _highlights;
 	_midtones = mid;
@@ -992,8 +1087,8 @@ void on_histoMidEntry_activate(GtkEntry *entry, gpointer user_data) {
 }
 
 void on_histoShadEntry_activate(GtkEntry *entry, gpointer user_data) {
-	double lo = atof(gtk_entry_get_text(entry));
-	if (lo <= 0.0) lo = 0.0;
+	float lo = atof(gtk_entry_get_text(entry));
+	if (lo <= 0.f) lo = 0.f;
 	if (lo >= _highlights) lo = _highlights;
 	_shadows = lo;
 	set_cursor_waiting(TRUE);
@@ -1006,9 +1101,9 @@ void on_histoShadEntry_activate(GtkEntry *entry, gpointer user_data) {
 }
 
 void on_histoHighEntry_activate(GtkEntry *entry, gpointer user_data) {
-	double hi = atof(gtk_entry_get_text(entry));
+	float hi = atof(gtk_entry_get_text(entry));
 	if (hi <= _shadows) hi = _shadows;
-	if (hi >= 1.0) hi = 1.0;
+	if (hi >= 1.f) hi = 1.f;
 	_highlights = hi;
 	set_cursor_waiting(TRUE);
 	update_histo_mtf();
@@ -1018,3 +1113,34 @@ void on_histoHighEntry_activate(GtkEntry *entry, gpointer user_data) {
 	g_free(str);
 	set_cursor_waiting(FALSE);
 }
+
+void apply_mtf_to_sequence(struct mtf_data *mtf_args) {
+	struct generic_seq_args *args = malloc(sizeof(struct generic_seq_args));
+	args->seq = mtf_args->seq;
+	args->force_float = FALSE;
+	args->partial_image = FALSE;
+	args->filtering_criterion = seq_filter_included;
+	args->nb_filtered_images = mtf_args->seq->selnum;
+	args->compute_size_hook = NULL;
+	args->prepare_hook = seq_prepare_hook;
+	args->finalize_hook = seq_finalize_hook;
+	args->save_hook = NULL;
+	args->image_hook = mtf_image_hook;
+	args->idle_function = NULL;
+	args->stop_on_error = FALSE;
+	args->description = _("Midtone Transfer Function");
+	args->has_output = TRUE;
+	args->new_seq_prefix = mtf_args->seqEntry;
+	args->load_new_sequence = TRUE;
+	args->force_ser_output = FALSE;
+	args->new_ser = NULL;
+	args->force_fitseq_output = FALSE;
+	args->new_fitseq = NULL;
+	args->user = mtf_args;
+	args->already_in_a_thread = FALSE;
+	args->parallel = TRUE;
+
+	mtf_args->fit = NULL;	// not used here
+
+	start_in_new_thread(generic_sequence_worker, args);
+}
