@@ -14,6 +14,7 @@ struct siril_dialog_data {
 void siril_message_dialog(GtkMessageType type, char *title, char *text);
 void queue_message_dialog(GtkMessageType type, char *title, char *text);
 void siril_data_dialog(GtkMessageType type, char *title, char *text, gchar *data);
-gboolean siril_confirm_dialog(gchar *title, gchar *msg, gboolean check_button);
+gboolean siril_confirm_dialog(gchar *title, gchar *msg);
+gboolean siril_confirm_dialog_and_remember(gchar *title, gchar *msg, gboolean *user_data);
 
 #endif
