@@ -14,6 +14,9 @@ struct savedial_data {
 
 enum {
 	PAGE_TIFF, PAGE_JPG, PAGE_FITS, PAGE_MISC
-} confirm;
+};
+
+void on_header_save_as_button_clicked();
+void on_header_save_button_clicked();
 
 #endif /* SRC_GUI_SAVE_DIALOG_H_ */
