@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2019 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -29,17 +29,18 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <string.h>
-#include <assert.h>
+#include <math.h>
 #ifdef _WIN32
 #include <io.h>
 #endif
-
 #include "core/siril.h"
 #include "core/proto.h"
 #include "gui/callbacks.h"
 #include "gui/progress_and_log.h"
 #include "algos/demosaicing.h"
-#include "io/ser.h"
+#include "io/conversion.h"
+#include "io/image_format_fits.h"
+#include "ser.h"
 
 static gboolean user_warned = FALSE;
 
@@ -221,6 +222,10 @@ static int ser_read_timestamp(struct ser_struct *ser_file) {
 	/* Check if file is large enough to have timestamps */
 	if (ser_file->filesize >= offset + (8 * ser_file->frame_count)) {
 		ser_file->ts = calloc(8, ser_file->frame_count);
+		if (!ser_file->ts) {
+			PRINT_ALLOC_ERR;
+			return 0;
+		}
 		ser_file->ts_alloc = ser_file->frame_count;
 
 		// Seek to start of timestamps
@@ -230,6 +235,8 @@ static int ser_read_timestamp(struct ser_struct *ser_file) {
 
 			if (8 != fread(&ser_file->ts[i], 1, 8, ser_file->file))
 				return 0;
+
+			ser_file->ts[i] = le64_to_cpu(ser_file->ts[i]);
 		}
 
 		/* Check order of Timestamps */
@@ -324,9 +331,20 @@ static int ser_read_header(struct ser_struct *ser_file) {
 	// modify this to support big endian
 	memcpy(&ser_file->lu_id, header + 14, 28);	// read all integers
 
+	ser_file->lu_id = le32_to_cpu(ser_file->lu_id);
+	ser_file->color_id = le32_to_cpu(ser_file->color_id);
+	ser_file->little_endian = le32_to_cpu(ser_file->little_endian);
+	ser_file->image_width = le32_to_cpu(ser_file->image_width);
+	ser_file->image_height = le32_to_cpu(ser_file->image_height);
+	ser_file->bit_pixel_depth = le32_to_cpu(ser_file->bit_pixel_depth);
+	ser_file->frame_count = le32_to_cpu(ser_file->frame_count);
+
 	memcpy(&ser_file->date, header + 162, 8);
 	memcpy(&ser_file->date_utc, header + 170, 8);
 
+	ser_file->date = le64_to_cpu(ser_file->date);
+	ser_file->date_utc = le64_to_cpu(ser_file->date_utc);
+
 	// strings
 	ser_file->file_id = g_strndup(header, 14);
 
@@ -379,12 +397,17 @@ static int ser_write_timestamps(struct ser_struct *ser_file) {
 			(int64_t)ser_file->byte_pixel_depth * (int64_t)ser_file->frame_count;
 
 		for (i = 0; i < ser_file->frame_count; i++) {
+			uint64_t ts;
+
 			if (i >= ser_file->ts_alloc)
 				break;
 			if ((int64_t)-1 == fseek64(ser_file->file, offset+(i*8), SEEK_SET)) {
 				return -1;
 			}
-			if (8 != fwrite(&ser_file->ts[i], 1, 8, ser_file->file)) {
+
+			ts = cpu_to_le64(ser_file->ts[i]);
+
+			if (8 != fwrite(&ts, 1, 8, ser_file->file)) {
 				perror("write timestamps:");
 				return -1;
 			}
@@ -396,6 +419,7 @@ static int ser_write_timestamps(struct ser_struct *ser_file) {
 /* (over)write the header of the opened file on the disk */
 static int ser_write_header(struct ser_struct *ser_file) {
 	char header[SER_HEADER_LEN];
+	struct ser_struct ser_file_le;
 
 	if (!ser_file || ser_file->file == NULL)
 		return -1;
@@ -404,15 +428,26 @@ static int ser_write_header(struct ser_struct *ser_file) {
 		return -1;
 	}
 
-	// modify this to support big endian
+	memcpy(&ser_file_le, ser_file, sizeof(struct ser_struct));
+
+	ser_file_le.lu_id = cpu_to_le32(ser_file_le.lu_id);
+	ser_file_le.color_id = cpu_to_le32(ser_file_le.color_id);
+	ser_file_le.little_endian = cpu_to_le32(ser_file_le.little_endian);
+	ser_file_le.image_width = cpu_to_le32(ser_file_le.image_width);
+	ser_file_le.image_height = cpu_to_le32(ser_file_le.image_height);
+	ser_file_le.bit_pixel_depth = cpu_to_le32(ser_file_le.bit_pixel_depth);
+	ser_file_le.frame_count = cpu_to_le32(ser_file_le.frame_count);
+	ser_file_le.date = cpu_to_le64(ser_file_le.date);
+	ser_file_le.date_utc = cpu_to_le64(ser_file_le.date_utc);
+
 	memset(header, 0, sizeof(header));
-	memcpy(header, ser_file->file_id, 14);
-	memcpy(header + 14, &ser_file->lu_id, 28);
-	memcpy(header + 42, ser_file->observer, 40);
-	memcpy(header + 82, ser_file->instrument, 40);
-	memcpy(header + 122, ser_file->telescope, 40);
-	memcpy(header + 162, &ser_file->date, 8);
-	memcpy(header + 170, &ser_file->date_utc, 8);
+	memcpy(header, ser_file_le.file_id, 14);
+	memcpy(header + 14, &ser_file_le.lu_id, 28);
+	memcpy(header + 42, ser_file_le.observer, 40);
+	memcpy(header + 82, ser_file_le.instrument, 40);
+	memcpy(header + 122, ser_file_le.telescope, 40);
+	memcpy(header + 162, &ser_file_le.date, 8);
+	memcpy(header + 170, &ser_file_le.date_utc, 8);
 
 	if (sizeof(header) != fwrite(header, 1, sizeof(header), ser_file->file)) {
 		perror("write");
@@ -422,7 +457,7 @@ static int ser_write_header(struct ser_struct *ser_file) {
 }
 
 /* populate fields that are not already set in ser_create_file */
-static void ser_write_header_from_fit(struct ser_struct *ser_file, fits *fit) {
+static int ser_write_header_from_fit(struct ser_struct *ser_file, fits *fit) {
 	ser_file->image_width = fit->rx;
 	ser_file->image_height = fit->ry;
 	fprintf(stdout, "setting SER image size as %dx%d\n", fit->rx, fit->ry);
@@ -442,6 +477,7 @@ static void ser_write_header_from_fit(struct ser_struct *ser_file, fits *fit) {
 		ser_file->bit_pixel_depth = 16;
 	} else {
 		siril_log_message(_("Writing to SER files from larger than 16-bit FITS images is not yet implemented\n"));
+		return 1;
 	}
 	if (fit->instrume[0] != 0) {
 		memset(ser_file->instrument, 0, 40);
@@ -458,6 +494,7 @@ static void ser_write_header_from_fit(struct ser_struct *ser_file, fits *fit) {
 
 	if (FITS_date_key_to_Unix_time(fit->date_obs, &ser_file->date_utc, &ser_file->date) == -1)
 		FITS_date_key_to_Unix_time(fit->date, &ser_file->date_utc, &ser_file->date);
+	return 0;
 }
 
 static int get_SER_Bayer_Pattern(ser_color pattern) {
@@ -480,7 +517,6 @@ static int get_SER_Bayer_Pattern(ser_color pattern) {
  * reorganized to match Siril's data format . */
 static void ser_manage_endianess_and_depth(struct ser_struct *ser_file,
 		WORD *data, int64_t frame_size) {
-	WORD pixel;
 	int i;
 	if (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_8) {
 		// inline conversion to 16 bit
@@ -489,9 +525,12 @@ static void ser_manage_endianess_and_depth(struct ser_struct *ser_file,
 	} else if (ser_file->little_endian == SER_BIG_ENDIAN) {
 		// inline conversion
 		for (i = frame_size - 1; i >= 0; i--) {
-			pixel = data[i];
-			pixel = (pixel >> 8) | (pixel << 8);
-			data[i] = pixel;
+			data[i] = be16_to_cpu(data[i]);
+		}
+	} else if (ser_file->little_endian == SER_LITTLE_ENDIAN) {
+		// inline conversion
+		for (i = frame_size - 1; i >= 0; i--) {
+			data[i] = le16_to_cpu(data[i]);
 		}
 	}
 }
@@ -536,6 +575,10 @@ void ser_convertTimeStamp(struct ser_struct *ser_file, GSList *timestamp) {
 	if (ser_file->ts)
 		free(ser_file->ts);
 	ser_file->ts = calloc(8, ser_file->frame_count);
+	if (!ser_file->ts) {
+		PRINT_ALLOC_ERR;
+		return;
+	}
 	ser_file->ts_alloc = ser_file->frame_count;
 
 	GSList *t = timestamp;
@@ -568,14 +611,22 @@ void ser_display_info(struct ser_struct *ser_file) {
 	fprintf(stdout, "========================================\n");
 }
 
+int ser_close_and_delete_file(struct ser_struct *ser_file) {
+	int retval;
+	char *filename = ser_file->filename;
+	ser_file->filename = NULL;
+	retval = ser_close_file(ser_file); // closes, frees and zeroes
+	siril_log_message(_("Removing failed SER file: %s\n"), filename);
+	g_unlink(filename);
+	free(filename);
+	return retval;
+}
+
 int ser_write_and_close(struct ser_struct *ser_file) {
 	if (ser_file == NULL) return -1;
 	if (!ser_file->frame_count) {
 		siril_log_color_message(_("The SER sequence is being created with no image in it.\n"), "red");
-		char *filename = ser_file->filename;
-		ser_file->filename = NULL;
-		ser_close_file(ser_file);// closes, frees and zeroes
-		g_unlink(filename);
+		ser_close_and_delete_file(ser_file);
 		return -1;
 	}
 	ser_write_header(ser_file);	// writes the header
@@ -646,7 +697,7 @@ int ser_compact_file(struct ser_struct *ser_file, unsigned char *successful_fram
 	return 0;
 }
 
-/* ser_file must be allocated
+/* ser_file must be allocated and initialised with ser_init_struct()
  * the file is created with no image size, the first image added will set it. */
 int ser_create_file(const char *filename, struct ser_struct *ser_file,
 		gboolean overwrite, struct ser_struct *copy_from) {
@@ -677,7 +728,10 @@ int ser_create_file(const char *filename, struct ser_struct *ser_file,
 
 		if (copy_from->ts && copy_from->frame_count > 0) {
 			ser_file->ts = calloc(8, copy_from->frame_count);
-			ser_file->ts_alloc = copy_from->frame_count;
+			if (!ser_file->ts) {
+				PRINT_ALLOC_ERR;
+			}
+			else ser_file->ts_alloc = copy_from->frame_count;
 		}
 		/* we write the header now, but it should be written again
 		 * before closing in case the number of the image in the new
@@ -751,6 +805,7 @@ int ser_close_file(struct ser_struct *ser_file) {
 }
 
 void ser_init_struct(struct ser_struct *ser_file) {
+	g_assert(ser_file);
 	memset(ser_file, 0, sizeof(struct ser_struct));
 }
 
@@ -801,12 +856,13 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit) {
 
 	fit->bitpix = (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_8) ? BYTE_IMG : USHORT_IMG;
 	fit->orig_bitpix = fit->bitpix;
+	fit->binning_x = fit->binning_y = 1;
 
 	/* If the user checks the SER CFA box, the video is opened in B&W
 	 * RGB and BGR are not coming from raw data. In consequence CFA does
 	 * not exist for these kind of cam */
 	ser_color type_ser = ser_file->color_id;
-	if (!com.debayer.open_debayer && type_ser != SER_RGB && type_ser != SER_BGR)
+	if (!com.pref.debayer.open_debayer && type_ser != SER_RGB && type_ser != SER_BGR)
 		type_ser = SER_MONO;
 
 	switch (type_ser) {
@@ -828,21 +884,21 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit) {
 		fit->naxes[2] = 3;
 		/* Get Bayer informations from header if available */
 		sensor_pattern sensortmp;
-		sensortmp = com.debayer.bayer_pattern;
-		if (com.debayer.use_bayer_header) {
+		sensortmp = com.pref.debayer.bayer_pattern;
+		if (com.pref.debayer.use_bayer_header) {
 			sensor_pattern bayer;
 			bayer = get_SER_Bayer_Pattern(type_ser);
-			if (bayer != com.debayer.bayer_pattern) {
-				if (bayer == BAYER_FILTER_NONE  && user_warned == FALSE) {
+			if (bayer != com.pref.debayer.bayer_pattern) {
+				if (bayer == BAYER_FILTER_NONE  && !user_warned) {
 					siril_log_color_message(_("No Bayer pattern found in the header file.\n"), "red");
 				}
 				else {
-					if (user_warned == FALSE) {
+					if (!user_warned) {
 						siril_log_color_message(_("Bayer pattern found in header (%s) is different"
 								" from Bayer pattern in settings (%s). Overriding settings.\n"),
-								"red", filter_pattern[bayer], filter_pattern[com.debayer.bayer_pattern]);
+								"salmon", filter_pattern[bayer], filter_pattern[com.pref.debayer.bayer_pattern]);
 					}
-					com.debayer.bayer_pattern = bayer;
+					com.pref.debayer.bayer_pattern = bayer;
 				}
 				user_warned = TRUE;
 			}
@@ -850,14 +906,18 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit) {
 		/* for performance consideration (and many others) we force the interpolation algorithm
 		 * to be BAYER_BILINEAR
 		 */
-		debayer(fit, BAYER_BILINEAR, FALSE);
-		com.debayer.bayer_pattern = sensortmp;
+		debayer(fit, BAYER_RCD, com.pref.debayer.bayer_pattern);
+		com.pref.debayer.bayer_pattern = sensortmp;
 		break;
 	case SER_BGR:
 		swap = 2;
 		/* no break */
 	case SER_RGB:
 		tmp = malloc(frame_size * sizeof(WORD));
+		if (!tmp) {
+			PRINT_ALLOC_ERR;
+			return -1;
+		}
 		memcpy(tmp, fit->data, sizeof(WORD) * frame_size);
 		fit->naxes[0] = fit->rx = ser_file->image_width;
 		fit->naxes[1] = fit->ry = ser_file->image_height;
@@ -893,6 +953,7 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit) {
 
 	fits_flip_top_to_bottom(fit);
 	fit->top_down = FALSE;
+
 	return 0;
 }
 
@@ -947,6 +1008,10 @@ static int read_area_from_image(struct ser_struct *ser_file, const int frame_no,
 		// allocated space is probably not enough to
 		// store whole lines or RGB data
 		read_buffer = malloc(read_size);
+		if (!read_buffer) {
+			PRINT_ALLOC_ERR;
+			return -1;
+		}
 	}
 	else read_buffer = outbuf;
 
@@ -1018,7 +1083,7 @@ int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 		return -1;
 
 	type_ser = ser_file->color_id;
-	if (!com.debayer.open_debayer &&
+	if (!com.pref.debayer.open_debayer &&
 			type_ser != SER_RGB && type_ser != SER_BGR)
 		type_ser = SER_MONO;
 
@@ -1039,21 +1104,21 @@ int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 		 * we extract one of the three channels and crop it to the requested area. */
 
 		/* Get Bayer informations from header if available */
-		sensortmp = com.debayer.bayer_pattern;
-		if (com.debayer.use_bayer_header) {
+		sensortmp = com.pref.debayer.bayer_pattern;
+		if (com.pref.debayer.use_bayer_header) {
 			sensor_pattern bayer;
 			bayer = get_SER_Bayer_Pattern(type_ser);
-			if (bayer != com.debayer.bayer_pattern) {
-				if (bayer == BAYER_FILTER_NONE && user_warned == FALSE) {
+			if (bayer != com.pref.debayer.bayer_pattern) {
+				if (bayer == BAYER_FILTER_NONE && !user_warned) {
 					siril_log_color_message(_("No Bayer pattern found in the header file.\n"), "red");
 				}
 				else {
-					if (user_warned == FALSE) {
+					if (!user_warned) {
 						siril_log_color_message(_("Bayer pattern found in header (%s) is different"
 								" from Bayer pattern in settings (%s). Overriding settings.\n"),
-								"red", filter_pattern[bayer], filter_pattern[com.debayer.bayer_pattern]);
+								"salmon", filter_pattern[bayer], filter_pattern[com.pref.debayer.bayer_pattern]);
 					}
-					com.debayer.bayer_pattern = bayer;
+					com.pref.debayer.bayer_pattern = bayer;
 				}
 				user_warned = TRUE;
 			}
@@ -1083,8 +1148,7 @@ int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 		 * to be BAYER_BILINEAR
 		 */
 		demosaiced_buf = debayer_buffer(rawbuf, &debayer_area.w,
-				&debayer_area.h, BAYER_BILINEAR, com.debayer.bayer_pattern,
-				NULL);
+				&debayer_area.h, BAYER_BILINEAR, com.pref.debayer.bayer_pattern);
 		free(rawbuf);
 		if (!demosaiced_buf)
 			return -1;
@@ -1093,19 +1157,20 @@ int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 		 * debayer_area is the demosaiced buf area.
 		 * xoffset and yoffset are the x,y offsets of area in the debayer area.
 		 */
+        const int nbpixels = debayer_area.w * debayer_area.h;
 		for (y = 0; y < area->h; y++) {
 			for (x = 0; x < area->w; x++) {
-				buffer[y*area->w + x] = demosaiced_buf[(yoffset+y)*debayer_area.w*3 + xoffset+x*3 + layer]; 
+				buffer[y*area->w + x] = demosaiced_buf[layer * nbpixels + (yoffset+y)*debayer_area.w + xoffset+x]; 
 			}
 		}
 
 		free(demosaiced_buf);
-		com.debayer.bayer_pattern = sensortmp;
+		com.pref.debayer.bayer_pattern = sensortmp;
 		break;
 
 	case SER_BGR:
 	case SER_RGB:
-		assert(ser_file->number_of_planes == 3);
+		g_assert(ser_file->number_of_planes == 3);
 		if (read_area_from_image(ser_file, frame_no, buffer, area, layer))
 			return -1;
 		ser_manage_endianess_and_depth(ser_file, buffer, area->w * area->h);
@@ -1120,7 +1185,7 @@ int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 
 int ser_read_opened_partial_fits(struct ser_struct *ser_file, int layer,
 		int frame_no, fits *fit, const rectangle *area) {
-	if (new_fit_image(&fit, area->w, area->h, 1))
+	if (new_fit_image(&fit, area->w, area->h, 1, DATA_USHORT))
 		return -1;
 	fit->top_down = TRUE;
 	if (ser_file->ts) {
@@ -1144,7 +1209,9 @@ int ser_write_frame_from_fit(struct ser_struct *ser_file, fits *fit, int frame_n
 		return -1;
 	if (ser_file->number_of_planes == 0) {
 		// adding first frame of a new sequence, use it to populate the header
-		ser_write_header_from_fit(ser_file, fit);
+		if (ser_write_header_from_fit(ser_file, fit)) {
+			return 1;
+		}
 	}
 	if (fit->rx != ser_file->image_width || fit->ry != ser_file->image_height) {
 		siril_log_message(_("Trying to add an image of different size in a SER\n"));
@@ -1160,10 +1227,16 @@ int ser_write_frame_from_fit(struct ser_struct *ser_file, fits *fit, int frame_n
 
 	if (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_8) {
 		data8 = malloc(frame_size * ser_file->byte_pixel_depth);
-		if (!data8) return -1;
+		if (!data8) {
+			PRINT_ALLOC_ERR;
+			return -1;
+		}
 	} else {
 		data16 = malloc(frame_size * ser_file->byte_pixel_depth);
-		if (!data16) return -1;
+		if (!data16) {
+			PRINT_ALLOC_ERR;
+			return -1;
+		}
 	}
 
 	for (plane = 0; plane < ser_file->number_of_planes; plane++) {
@@ -1171,7 +1244,7 @@ int ser_write_frame_from_fit(struct ser_struct *ser_file, fits *fit, int frame_n
 		for (pixel = 0; pixel < ser_file->image_width * ser_file->image_height;
 				pixel++) {
 			if (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_8)
-				data8[dest] = (BYTE)(fit->pdata[plane][pixel]);
+				data8[dest] = round_to_BYTE(fit->pdata[plane][pixel]);
 			else {
 				if (ser_file->little_endian == SER_BIG_ENDIAN)
 					data16[dest] = (fit->pdata[plane][pixel] >> 8 | fit->pdata[plane][pixel] << 8);
@@ -1240,9 +1313,141 @@ int64_t ser_compute_file_size(struct ser_struct *ser_file, int nb_frames) {
 		frame_size = (size - SER_HEADER_LEN) / ser_file->frame_count;
 		size = SER_HEADER_LEN + frame_size * nb_frames;
 	}
-	/* SER can be demosaiced on the fly on creation.
-	 * TODO: Is this the good test? */
-	if (ser_is_cfa(ser_file) && com.debayer.open_debayer)
-		size *= 3;
 	return size;
 }
+
+static GdkPixbufDestroyNotify free_preview_data(guchar *pixels, gpointer data) {
+	free(pixels);
+	free(data);
+	return FALSE;
+}
+
+/**
+ * Create a monochrome preview (only the first channel is displayed) of a SER file in a GdkPixbuf.
+ * @param filename
+ * @return a GdkPixbuf containing the preview or NULL
+ */
+GdkPixbuf* get_thumbnail_from_ser(char *filename, gchar **descr) {
+	GdkPixbuf *pixbuf = NULL;
+	int MAX_SIZE = com.pref.thumbnail_size;
+	gchar *description = NULL;
+	int i, j, k, l, N, M;
+	int w, h, pixScale, Ws, Hs, n_channels, n_frames, bit;
+	int sz;
+	struct ser_struct ser;
+	fits fit = { 0 };
+
+	ser_init_struct(&ser);
+	if (ser_open_file(filename, &ser)) {
+		return NULL;
+	}
+	float *pix = malloc(MAX_SIZE * sizeof(float));
+	float *ima_data = NULL, *ptr, byte, n, m, max, min, wd, avr;
+	guchar *pixbuf_data = NULL, *pptr;
+
+	w = ser.image_width;
+	h = ser.image_height;
+	sz = w * h;
+	ima_data = malloc(sz * sizeof(float));
+	pixbuf_data = malloc(3 * MAX_SIZE * MAX_SIZE * sizeof(guchar));
+
+	ser_read_frame(&ser, 0, &fit);
+
+	for (i = 0; i < sz; i++) {
+		ima_data[i + 0] = (float)fit.pdata[RLAYER][i];
+	}
+
+	i = (int) ceil((float) w / MAX_SIZE);
+	j = (int) ceil((float) h / MAX_SIZE);
+	pixScale = (i > j) ? i : j;	// picture scale factor
+	if (pixScale == 0) return NULL;
+	Ws = w / pixScale; 			// picture width in pixScale blocks
+	Hs = h / pixScale; 			// -//- height pixScale
+
+	n_frames = ser.frame_count;
+	bit = ser.bit_pixel_depth;
+
+	switch (ser.color_id) {
+	case SER_MONO:
+		n_channels = 1;
+		break;
+	default:
+		n_channels = 3;
+	}
+
+	description = g_strdup_printf("%d x %d %s\n%d %s (%d bits)\n%d %s\n%s", w,
+			h, ngettext("pixel", "pixels", h), n_channels,
+			ngettext("channel", "channels", n_channels), bit, n_frames,
+			ngettext("frame", "frames", n_frames), _("(Monochrome Preview)"));
+
+	M = 0; // line number
+	for (i = 0; i < Hs; i++) { // cycle through a blocks by lines
+		//pptr = &pixbuf_data[i * Ws * 3];
+		for (j = 0; j < MAX_SIZE; j++)
+			pix[j] = 0;
+		m = 0.f; // amount of strings read in block
+		for (l = 0; l < pixScale; l++, m++) { // cycle through a block lines
+			ptr = &ima_data[M * w];
+			N = 0; // number of column
+			for (j = 0; j < Ws; j++) { // cycle through a blocks by columns
+				n = 0.;	// amount of columns read in block
+				byte = 0.; // average intensity in block
+				for (k = 0; k < pixScale; k++, n++) { // cycle through block pixels
+					if (N++ < w) // row didn't end
+						byte += *ptr++; // sum[(pix-min)/wd]/n = [sum(pix)/n-min]/wd
+					else
+						break;
+				}
+				pix[j] += byte / n; //(byte / n - min)/wd;
+			}
+			if (++M >= h)
+				break;
+		}
+		// fill unused picture pixels
+		ptr = &ima_data[i * Ws];
+		for (l = 0; l < Ws; l++)
+			*ptr++ = pix[l] / m;
+	}
+	ptr = ima_data;
+	sz = Ws * Hs;
+	max = min = *ptr;
+	avr = 0;
+	for (i = 0; i < sz; i++, ptr++) {
+		float tmp = *ptr;
+		if (tmp > max)
+			max = tmp;
+		else if (tmp < min)
+			min = tmp;
+		avr += tmp;
+	}
+	avr /= (float) sz;
+	wd = max - min;
+	avr = (avr - min) / wd;	// normal average by preview
+	if (avr > 1.)
+		wd /= avr;
+	ptr = ima_data;
+	for (i = Hs - 1; i > -1; i--) {	// fill pixbuf mirroring image by vertical
+		pptr = &pixbuf_data[Ws * i * 3];
+		for (j = 0; j < Ws; j++) {
+			*pptr++ = (guchar) round_to_BYTE(255.f * (*ptr - min) / wd);
+			*pptr++ = (guchar) round_to_BYTE(255.f * (*ptr - min) / wd);
+			*pptr++ = (guchar) round_to_BYTE(255.f * (*ptr - min) / wd);
+			ptr++;
+		}
+	}
+
+	ser_close_file(&ser);
+	pixbuf = gdk_pixbuf_new_from_data(pixbuf_data,		// guchar* data
+			GDK_COLORSPACE_RGB,	// only this supported
+			FALSE,				// no alpha
+			8,				// number of bits
+			Ws, Hs,				// size
+			Ws * 3,				// line length in bytes
+			(GdkPixbufDestroyNotify) free_preview_data, // function (*GdkPixbufDestroyNotify) (guchar *pixels, gpointer data);
+			NULL
+			);
+	free(ima_data);
+	free(pix);
+	*descr = description;
+	return pixbuf;
+}
