@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2019 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -22,18 +22,22 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
-#include <assert.h>
-#include "registration/registration.h"
+
 #include "core/siril.h"
-#include "core/processing.h"
 #include "core/proto.h"
+#include "core/processing.h"
+#include "core/OS_utils.h"
 #include "algos/star_finder.h"
+#include "algos/statistics.h"
 #include "algos/PSF.h"
+#include "gui/image_display.h"
 #include "gui/PSF_list.h"
 #include "gui/progress_and_log.h"
 #include "gui/callbacks.h"
 #include "io/sequence.h"
 #include "io/ser.h"
+#include "io/image_format_fits.h"
+#include "registration/registration.h"
 #include "registration/matching/atpmatch.h"
 #include "registration/matching/match.h"
 #include "registration/matching/misc.h"
@@ -82,12 +86,13 @@ static int star_align_prepare_hook(struct generic_seq_args *args) {
 	}
 
 	/* first we're looking for stars in reference image */
-	if (seq_read_frame(args->seq, regargs->reference_image, &fit)) {
+	if (seq_read_frame(args->seq, regargs->reference_image, &fit, FALSE, -1)) {
 		siril_log_message(_("Could not load reference image\n"));
 		args->seq->regparam[regargs->layer] = NULL;
 		free(sadata->current_regdata);
 		return 1;
 	}
+
 	siril_log_color_message(_("Reference Image:\n"), "green");
 
 	if (regargs->matchSelection && regargs->selection.w > 0 && regargs->selection.h > 0) {
@@ -135,8 +140,17 @@ static int star_align_prepare_hook(struct generic_seq_args *args) {
 	 */
 	i = 0;
 	sadata->refstars = malloc((MAX_STARS + 1) * sizeof(fitted_PSF *));
+	if (!sadata->refstars) {
+		PRINT_ALLOC_ERR;
+		return 1;
+	}
 	while (i < MAX_STARS && com.stars[i]) {
 		fitted_PSF *tmp = malloc(sizeof(fitted_PSF));
+		if (!tmp) {
+			PRINT_ALLOC_ERR;
+			sadata->refstars[i] = NULL;
+			return 1;
+		}
 		memcpy(tmp, com.stars[i], sizeof(fitted_PSF));
 		sadata->refstars[i] = tmp;
 		sadata->refstars[i+1] = NULL;
@@ -152,21 +166,29 @@ static int star_align_prepare_hook(struct generic_seq_args *args) {
 	FWHM_average(sadata->refstars, sadata->fitted_stars, &FWHMx, &FWHMy, &units);
 	siril_log_message(_("FWHMx:%*.2f %s\n"), 12, FWHMx, units);
 	siril_log_message(_("FWHMy:%*.2f %s\n"), 12, FWHMy, units);
-	sadata->current_regdata[regargs->reference_image].fwhm = FWHMx;
 	sadata->current_regdata[regargs->reference_image].roundness = FWHMy/FWHMx;
+	sadata->current_regdata[regargs->reference_image].fwhm = FWHMx;
+	sadata->current_regdata[regargs->reference_image].weighted_fwhm = FWHMx;
 	
 	if (!regargs->translation_only) {
 		// allocate destination sequence data
 		regargs->imgparam = calloc(args->nb_filtered_images, sizeof(imgdata));
 		regargs->regparam = calloc(args->nb_filtered_images, sizeof(regdata));
+		if (!regargs->imgparam  || !regargs->regparam) {
+			PRINT_ALLOC_ERR;
+			return 1;
+		}
 	}
 
 	if (args->seq->type == SEQ_SER) {
-		/* copied from ser_prepare_hook with one variation */
-		char dest[256];
-
+		/* copied from seq_prepare_hook with one variation */
 		args->new_ser = malloc(sizeof(struct ser_struct));
+		if (!args->new_ser) {
+			PRINT_ALLOC_ERR;
+			return 1;
+		}
 
+		char dest[256];
 		const char *ptr = strrchr(args->seq->seqname, G_DIR_SEPARATOR);
 		if (ptr)
 			snprintf(dest, 255, "%s%s.ser", regargs->prefix, ptr + 1);
@@ -185,8 +207,16 @@ static int star_align_prepare_hook(struct generic_seq_args *args) {
 			return 1;
 		}
 	}
+	else if (args->seq->type == SEQ_FITSEQ) {
+		if (seq_prepare_hook(args))
+			return 1;
+	}
 
 	sadata->success = calloc(args->nb_filtered_images, sizeof(BYTE));
+	if (!sadata->success) {
+		PRINT_ALLOC_ERR;
+		return 1;
+	}
 	return 0;
 }
 
@@ -214,7 +244,7 @@ static int star_align_image_hook(struct generic_seq_args *args, int out_index, i
 
 	if (in_index != regargs->reference_image) {
 		fitted_PSF **stars;
-		if (args->seq->type == SEQ_SER) {
+		if (args->seq->type == SEQ_SER || args->seq->type == SEQ_FITSEQ) {
 			siril_log_color_message(_("Frame %d:\n"), "bold", filenum);
 		}
 		if (regargs->matchSelection && regargs->selection.w > 0 && regargs->selection.h > 0) {
@@ -257,18 +287,30 @@ static int star_align_image_hook(struct generic_seq_args *args, int out_index, i
 		}
 
 		FWHM_average(stars, nbpoints, &FWHMx, &FWHMy, &units);
+#ifdef _OPENMP
 #pragma omp critical
+#endif
 		print_alignment_results(H, filenum, FWHMx, FWHMy, units);
-		sadata->current_regdata[in_index].fwhm = FWHMx;
+
 		sadata->current_regdata[in_index].roundness = FWHMy/FWHMx;
+		sadata->current_regdata[in_index].fwhm =  FWHMx;
+		sadata->current_regdata[in_index].weighted_fwhm = 2 * FWHMx
+				* (((double) sadata->fitted_stars) - (double) H.pair_matched)
+				/ (double) sadata->fitted_stars + FWHMx;
 
 		if (!regargs->translation_only) {
 			if (regargs->x2upscale) {
-				cvResizeGaussian(fit, fit->rx * 2, fit->ry * 2, OPENCV_NEAREST);
+				if (cvResizeGaussian(fit, fit->rx * 2, fit->ry * 2, OPENCV_NEAREST)) {
+					free_fitted_stars(stars);
+					return 1;
+				}
 				cvApplyScaleToH(&H, 2.0);
 			}
 			fits_flip_top_to_bottom(fit);
-			cvTransformImage(fit, (long) sadata->ref.x, (long) sadata->ref.y, H, regargs->interpolation);
+			if (cvTransformImage(fit, (long) sadata->ref.x, (long) sadata->ref.y, H, regargs->interpolation)) {
+				free_fitted_stars(stars);
+				return 1;
+			}
 			fits_flip_top_to_bottom(fit);
 		}
 
@@ -276,7 +318,8 @@ static int star_align_image_hook(struct generic_seq_args *args, int out_index, i
 	}
 	else {
 		if (regargs->x2upscale && !regargs->translation_only) {
-			cvResizeGaussian(fit, fit->rx * 2, fit->ry * 2, OPENCV_NEAREST);
+			if (cvResizeGaussian(fit, fit->rx * 2, fit->ry * 2, OPENCV_NEAREST))
+				return 1;
 		}
 	}
 
@@ -284,10 +327,12 @@ static int star_align_image_hook(struct generic_seq_args *args, int out_index, i
 		regargs->imgparam[out_index].filenum = args->seq->imgparam[in_index].filenum;
 		regargs->imgparam[out_index].incl = SEQUENCE_DEFAULT_INCLUDE;
 		regargs->regparam[out_index].fwhm = sadata->current_regdata[in_index].fwhm;	// not FWHMx because of the ref frame
+		regargs->regparam[out_index].weighted_fwhm = sadata->current_regdata[in_index].weighted_fwhm;
 		regargs->regparam[out_index].roundness = sadata->current_regdata[in_index].roundness;
+		invalidate_stats_from_fit(fit);
 	} else {
-		set_shifts(args->seq, in_index, regargs->layer, (float)H.h02, (float)-H.h12,
-				fit->top_down);
+		set_shifts(args->seq, in_index, regargs->layer, (float) H.h02,
+				(float) -H.h12, fit->top_down);
 		args->seq->imgparam[out_index].incl = SEQUENCE_DEFAULT_INCLUDE;
 	}
 	sadata->success[out_index] = 1;
@@ -313,7 +358,7 @@ static int star_align_finalize_hook(struct generic_seq_args *args) {
 			int j;
 			for (i = 0, j = 0; i < regargs->new_total; i++, j++) {
 				while (!sadata->success[j] && j < args->nb_filtered_images) j++;
-				assert(sadata->success[j]);
+				g_assert(sadata->success[j]);
 				if (i != j) {
 					regargs->imgparam[i] = regargs->imgparam[j];
 					regargs->regparam[i] = regargs->regparam[j];
@@ -330,24 +375,37 @@ static int star_align_finalize_hook(struct generic_seq_args *args) {
 			ser_write_and_close(args->new_ser);
 			free(args->new_ser);
 		}
-
+		else if (args->seq->type == SEQ_FITSEQ) {
+			seq_finalize_hook(args);
+		}
 	} else {
 		regargs->new_total = 0;
 		free(args->seq->regparam[regargs->layer]);
 		args->seq->regparam[regargs->layer] = NULL;
+
+		// args->new_ser can be null if stars were not detected in the reference image
+		if (args->seq->type == SEQ_SER && args->new_ser) {
+			ser_close_and_delete_file(args->new_ser);
+			free(args->new_ser);
+		}
+		else if (args->seq->type == SEQ_FITSEQ && args->new_fitseq) {
+			fitseq_close_and_delete_file(args->new_fitseq);
+			free(args->new_fitseq);
+		}
 	}
+
 	if (sadata->success) free(sadata->success);
 	free(sadata);
 	args->user = NULL;
 
-	update_used_memory();
-
 	if (!args->retval) {
 		siril_log_message(_("Registration finished.\n"));
-		siril_log_color_message(_("%d images processed.\n"), "green",
-				args->nb_filtered_images);
-		siril_log_color_message(_("Total: %d failed, %d registered.\n"), "green",
-				failed, regargs->new_total);
+		gchar *str = ngettext("%d image processed.\n", "%d images processed.\n", args->nb_filtered_images);
+		str = g_strdup_printf(str, args->nb_filtered_images);
+		siril_log_color_message(str, "green");
+		siril_log_color_message(_("Total: %d failed, %d registered.\n"), "green", failed, regargs->new_total);
+
+		g_free(str);
 		if (!regargs->translation_only) {
 			// explicit sequence creation to copy imgparam and regparam
 			create_output_sequence_for_global_star(regargs);
@@ -368,6 +426,7 @@ static int star_align_finalize_hook(struct generic_seq_args *args) {
 int register_star_alignment(struct registration_args *regargs) {
 	struct generic_seq_args *args = malloc(sizeof(struct generic_seq_args));
 	args->seq = regargs->seq;
+	args->force_float = FALSE;
 	args->partial_image = FALSE;
 	if (regargs->process_all_frames) {
 		args->filtering_criterion = seq_filter_all;
@@ -376,6 +435,7 @@ int register_star_alignment(struct registration_args *regargs) {
 		args->filtering_criterion = seq_filter_included;
 		args->nb_filtered_images = regargs->seq->selnum;
 	}
+	args->compute_size_hook = NULL;
 	args->prepare_hook = star_align_prepare_hook;
 	args->image_hook = star_align_image_hook;
 	args->save_hook = NULL;
@@ -384,9 +444,14 @@ int register_star_alignment(struct registration_args *regargs) {
 	args->stop_on_error = FALSE;
 	args->description = _("Global star registration");
 	args->has_output = !regargs->translation_only;
+	args->output_type = get_data_type(args->seq->bitpix);
+	args->upscale_ratio = regargs->x2upscale ? 2.0 : 1.0;
 	args->new_seq_prefix = regargs->prefix;
 	args->load_new_sequence = TRUE;
 	args->force_ser_output = FALSE;
+	args->new_ser = NULL;
+	args->force_fitseq_output = FALSE;
+	args->new_fitseq = NULL;
 	args->already_in_a_thread = TRUE;
 	args->parallel = TRUE;
 
@@ -446,7 +511,10 @@ static void print_alignment_results(Homography H, int filenum, float FWHMx, floa
 
 	/* Matching information */
 	siril_log_color_message(_("Matching stars in image %d: done\n"), "green", filenum);
-	siril_log_message(_("%d pair matches.\n"), H.pair_matched);
+	gchar *str = ngettext("%d pair match.\n", "%d pair matches.\n", H.pair_matched);
+	str = g_strdup_printf(str, H.pair_matched);
+	siril_log_message(str);
+	g_free(str);
 	inliers = 1.0 - ((((double) H.pair_matched - (double) H.Inliers)) / (double) H.pair_matched);
 	siril_log_message(_("Inliers:%*.3f\n"), 11, inliers);
 
